#!/usr/bin/env python

import modshogun
import pickle
import os
import filecmp
import numpy
import sys
import difflib

from generator import setup_tests, get_fname, blacklist, get_test_mod, run_test

def typecheck(a, b):
	if str(type(a)).find('shogun')>=0 and \
		str(type(a)).find('Labels')>=0 and \
		str(type(b)).find('shogun')>=0 and \
		str(type(b)).find('Labels')>=0:
		 return True
	return type(a) == type(b)


def compare(a, b, tolerance):
	if not typecheck(a,b): return False

	if type(a) == numpy.ndarray:
		if tolerance:
			return numpy.max(numpy.abs(a - b)) < tolerance
		else:
			return numpy.all(a == b)
	elif isinstance(a, modshogun.SGObject):
		# old binary way to compare, via serialization and string comparison
		#return pickle.dumps(a) == pickle.dumps(b)

		# new, parameter framework based comparison up to tolerance
		shogun_tolerance = 1e-5 if tolerance is None else tolerance
		result = a.equals(b, shogun_tolerance)

		# print debug output in case of failure
		if not result:
			print "Equals failed with debug output"
			old_loglevel=a.io.get_loglevel()
			a.io.set_loglevel(1)
			a.equals(b, shogun_tolerance)
			a.io.set_loglevel(old_loglevel)

		return result
	elif type(a) in (tuple,list):
		if len(a) != len(b): return False
		for obj1, obj2 in zip(a,b):
			if not compare(obj1, obj2, tolerance): return False
		return True

	return a == b

def compare_dbg(a, b, tolerance):
	if not compare_dbg_helper(a, b, tolerance):
		import pdb
		pdb.set_trace()

def compare_dbg_helper(a, b, tolerance):
	if not typecheck(a,b):
		print("Type mismatch (type(a)=%s vs type(b)=%s)" % (str(type(a)),str(type(b))))
		return False

	if type(a) == numpy.ndarray:
		if tolerance:
			if numpy.max(numpy.abs(a - b)) < tolerance:
				return True
			else:
				print("Numpy Array mismatch > max_tol")
				print(a-b)
				return False
		else:
			if numpy.all(a == b):
				return True
			else:
				print("Numpy Array mismatch")
				print(a-b)
				return False
	elif isinstance(a, modshogun.SGObject):
		if pickle.dumps(a) == pickle.dumps(b):
			return True
		print("a", pickle.dumps(a))
		print("b", pickle.dumps(b))
		return False
	elif type(a) in (tuple,list):
		if len(a) != len(b):
			print("Length mismatch (len(a)=%d vs len(b)=%d)" % (len(a), len(b)))
			return False
		for obj1, obj2 in zip(a,b):
			if not compare_dbg(obj1, obj2, tolerance):
				return False
		return True

	if (a==b):
		return True
	else:
		print("a!=b")
		print("a", a)
		print("b", b)
		return False

def get_fail_string(a):
	failed_string = []
	if type(a) in (tuple,list):
		for i in range(len(a)):
			failed_string.append(get_fail_string(a[i]))
	elif isinstance(a, modshogun.SGObject):
		failed_string.append(pickle.dumps(a))
	else:
		failed_string.append(str(a))
	return failed_string

def get_split_string(a):
	strs=[]
	for l in a:
		if type(l) in (tuple,list):
			e=l[0]
		else:
			e=l
		strs.extend(e.replace('\\n','\n').splitlines())
	return strs

def tester(tests, cmp_method, tolerance, failures, missing):
	failed=[]

	for t in tests:
		try:
			mod, mod_name = get_test_mod(t)
			n=len(mod.parameter_list)
		except TypeError:
			continue
		except Exception as e:
			print("%-60s ERROR (%s)" % (t,e))
			failed.append(t)
			continue
		fname = ""

		for i in range(n):
			fname = get_fname(mod_name, i)
			setting_str = "%s setting %d/%d" % (t,i+1,n)
			try:
				a = run_test(mod, mod_name, i)
				b = pickle.load(open(fname))

				try:
					if cmp_method(a,b,tolerance):
						if not failures and not missing:
							print("%-60s OK" % setting_str)
					else:
						if not missing:
							failed.append((setting_str, get_fail_string(a), get_fail_string(b)))
							print("%-60s ERROR" % setting_str)
				except Exception as e:
					print(setting_str, e)
			except IOError as e:
				if not failures:
					print("%-60s NO TEST (%s)" % (setting_str, e))
			except Exception as e:
				failed.append(setting_str)
				if not missing:
					print("%-60s EXCEPTION %s" % (setting_str,e))
	return failed

if __name__=='__main__':
	import sys
	from optparse import OptionParser
	op=OptionParser()
	op.add_option("-d", "--debug", action="store_true", default=False,
				help="detailed debug output of objects that don't match")
	op.add_option("-f", "--failures", action="store_true", default=False,
				help="show only failures")
	op.add_option("-m", "--missing", action="store_true", default=False,
				help="show only missing tests")
	op.add_option("-t", "--tolerance", action="store", default=None,
	              help="tolerance used to estimate accuracy")

	op.set_usage("[<file1> <file2> ...]")
	(opts, args)=op.parse_args()
	if opts.debug:
		cmp_method=compare_dbg
	else:
		cmp_method=compare
	tests = setup_tests(args)
	failed = tester(tests, cmp_method, opts.tolerance, opts.failures, opts.missing)
	if failed:
		print()
		print("The following tests failed!")
		for f in failed:
			print("\t", f[0])

		print()
		print("Detailled failures:")
		print()
		for f in failed:
			print("\t", f[0])
			got=get_split_string(f[1])
			expected=get_split_string(f[2])
			#print "=== EXPECTED =========="
			#import pdb
			#pdb.set_trace()
			#print '\n'.join(expected)
			#print "=== GOT ==============="
			#print '\n'.join(got)
			print("====DIFF================")
			print('\n'.join(difflib.unified_diff(expected, got, fromfile='expected', tofile='got')))
			print("====EOT================")
			print("\n\n\n")

		sys.exit(1)
	sys.exit(0)



#!/usr/bin/env python
from tools.load import LoadMatrix
from numpy import random
lm=LoadMatrix()

ground_truth = lm.load_labels('../data/label_train_twoclass.dat')
random.seed(17)
predicted = random.randn(len(ground_truth))

parameter_list = [[ground_truth,predicted]]

def evaluation_contingencytableevaluation_modular (ground_truth, predicted):
	from modshogun import BinaryLabels
	from modshogun import ContingencyTableEvaluation
	from modshogun import AccuracyMeasure,ErrorRateMeasure,BALMeasure
	from modshogun import WRACCMeasure,F1Measure,CrossCorrelationMeasure
	from modshogun import RecallMeasure,PrecisionMeasure,SpecificityMeasure

	ground_truth_labels = BinaryLabels(ground_truth)
	predicted_labels = BinaryLabels(predicted)

	base_evaluator = ContingencyTableEvaluation()
	base_evaluator.evaluate(predicted_labels,ground_truth_labels)

	evaluator = AccuracyMeasure()
	accuracy = evaluator.evaluate(predicted_labels,ground_truth_labels)

	evaluator = ErrorRateMeasure()
	errorrate = evaluator.evaluate(predicted_labels,ground_truth_labels)

	evaluator = BALMeasure()
	bal = evaluator.evaluate(predicted_labels,ground_truth_labels)

	evaluator = WRACCMeasure()
	wracc = evaluator.evaluate(predicted_labels,ground_truth_labels)

	evaluator = F1Measure()
	f1 = evaluator.evaluate(predicted_labels,ground_truth_labels)

	evaluator = CrossCorrelationMeasure()
	crosscorrelation = evaluator.evaluate(predicted_labels,ground_truth_labels)

	evaluator = RecallMeasure()
	recall = evaluator.evaluate(predicted_labels,ground_truth_labels)

	evaluator = PrecisionMeasure()
	precision = evaluator.evaluate(predicted_labels,ground_truth_labels)

	evaluator = SpecificityMeasure()
	specificity = evaluator.evaluate(predicted_labels,ground_truth_labels)

	return accuracy, errorrate, bal, wracc, f1, crosscorrelation, recall, precision, specificity


if __name__=='__main__':
	print('EvaluationContingencyTableEvaluation')
	evaluation_contingencytableevaluation_modular(*parameter_list[0])




#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Written (W) 2006-2007 Soeren Sonnenburg
# Written (W) 2006-2007 Mikio Braun
# Copyright (C) 2007 Fraunhofer Institute FIRST and Max-Planck-Society
#

import time
from string import maketrans

""" this function is 100% compatible to the matlab function, thus it is one based (!)
	use one_based=False if needed, then however the interval is [start,stop) (excluding stop)
"""
def load_genomic(chromosome, strand, start, stop, genome, one_based=True):
	fname = '/fml/ag-raetsch/share/databases/genomes/' + genome + '/' + chromosome[3:] + '.flat'
	f=file(fname)
	if one_based:
		f.seek(start-1)
		str=f.read(stop-start+1)
	else:
		f.seek(start)
		str=f.read(stop-start)

	if strand=='-':
		return reverse_complement(str)
	elif strand=='+':
		return str
	else:
		print 'strand must be + or -'
		raise KeyError

""" read a table browser ascii output file (http://genome.ucsc.edu/cgi-bin/hgTables) """
def read_table_browser(f):
	table=dict();
	for l in f.readlines():
		if not l.startswith('#'):
			(name,chrom,strand,txStart,txEnd,cdsStart,cdsEnd,exonCount,exonStarts,exonEnds,proteinID,alignID)=l.split('\t')
			exonStarts=[ int(i) for i in exonStarts.split(',')[:-1] ]
			exonEnds=[ int(i) for i in exonEnds.split(',')[:-1] ]

			table[name]={ 'chrom': chrom, 'strand': strand, 'txStart': int(txStart), 'txEnd': int(txEnd),
			'cdsStart': int(cdsStart), 'cdsEnd': int(cdsEnd), 'exonCount': int(exonCount), 'exonStarts': exonStarts,
			'exonEnds': exonEnds, 'proteinID': proteinID, 'alignID': alignID[:-1] }

	return table

""" get promoter region """
def get_promoter_region(chromosome, strand, gene_start, gene_end, genome, length):

	if strand == '+':
		return load_genomic(chromosome, strand, gene_start, gene_start+length, genome, one_based=False)
	elif strand == '-':
		return load_genomic(chromosome, strand, gene_end, gene_end+length, genome, one_based=False)
	else:
		print 'unknown strand'
		return None

""" reverse + complement a DNA sequence (only letters ACGT are translated!)
	FIXME won't work with all the rest like y... """
def reverse_complement(str):
	t=maketrans('acgtACGT','tgcaTGCA')
	return str[len(str)::-1].translate(t)

""" works only with .fa files that contain a single entry """
def read_single_fasta(fname):
	str=file(fname).read()
	str=str[str.index('\n')+1:].replace('\n','')
	return str

""" writes only single enty .fa files """
def write_single_fasta(fname, name, str, linelen=60):
	header= '>' + name + '\n'
	f=file(fname,'a')
	f.write(header)
	for i in xrange(0,len(str),linelen):
		f.write(str[i:i+linelen]+'\n')
	f.close()

""" read fasta as dictionary """
def read_fasta(f):
	fasta=dict()

	for s in f.readlines():
		if s.startswith('>'):
			key=s[1:-1]
			fasta[key]=""
		else:
			fasta[key]+=s[:-1]

	return fasta

""" write dictionary fasta """
def write_fasta(f, d, linelen=60):
    for k in sorted(d):
        f.write('>%s\n' % k);
        s = d[k]
        for i in xrange(0, len(s), linelen):
            f.write(s[i:i+linelen] + '\n')

def write_gff(f, (source, version), (seqtype, seqname), descrlist, skipheader=False):
	""" writes a gff version 2 file
		descrlist is a list of dictionaries, each of which contain these fields:
		<seqname> <source> <feature> <start> <end> <score> <strand> <frame> [attributes] [comments]
	"""

	if not skipheader:
		f.write('##gff-version 2\n')
		f.write('##source-version %s %s\n' % (source, version) )

		t=time.localtime()
		f.write("##date %d-%d-%d %d:%d:%d\n" % t[0:6])

	f.write('##Type %s %s\n' % (seqtype, seqname) )

	for d in descrlist:
		f.write('%s\t%s\t%s\t%d\t%d\t%f\t%s\t%d' % (d['seqname'], d['source'],
											d['feature'], d['start'], d['end'],
											d['score'], d['strand'], d['frame']))
		if d.has_key('attributes'):
			f.write('\t' + d['attributes'])
			if d.has_key('comments'):
				f.write('\t' + d['comments'])
		f.write('\n')


if __name__ == '__main__':
	import sys,os

	table=read_table_browser(file('/fml/ag-raetsch/home/sonne/addnet/tfbs/share/data/wt1_bibliosphere_table_browser_hg17.txt'))
	print table.keys()
	print table[table.keys()[0]]
	d = { 'ahoernchen' : 'ACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGT',
		  'bhoernchen' : 'GATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACA' }

	write_fasta(sys.stdout, d)
	write_fasta(file('/tmp/test.fa','w'), d)

	d2 = read_fasta(file('/tmp/test.fa'))
	os.unlink('/tmp/test.fa')

	print d
	print d2
	print d == d2

	p=load_genomic('chr5', '+', 100000, 100100,'hg17')
	n=load_genomic('chr1', '-', 3000000, 3001000,'mm7')
	write_single_fasta('bla.fa','bla', 'ACGT')
	n2=read_single_fasta('bla.fa')



#!/usr/bin/env python
from tools.load import LoadMatrix
lm = LoadMatrix()
fm_train_real = lm.load_numbers('../data/fm_train_real.dat')
fm_test_real = lm.load_numbers('../data/fm_test_real.dat')
label_train_multiclass = lm.load_labels('../data/label_train_multiclass.dat')

parameter_list=[
		[ fm_train_real, fm_test_real, label_train_multiclass, 1.2, 1.2, 1e-5, 1, 0.001, 1.5],
		[ fm_train_real, fm_test_real, label_train_multiclass, 5, 1.2, 1e-2, 1, 0.001, 2]]

def mkl_multiclass_modular (fm_train_real, fm_test_real, label_train_multiclass,
	width, C, epsilon, num_threads, mkl_epsilon, mkl_norm):

	from modshogun import CombinedFeatures, RealFeatures, MulticlassLabels
	from modshogun import CombinedKernel, GaussianKernel, LinearKernel,PolyKernel
	from modshogun import MKLMulticlass

	kernel = CombinedKernel()
	feats_train = CombinedFeatures()
	feats_test = CombinedFeatures()

	subkfeats_train = RealFeatures(fm_train_real)
	subkfeats_test = RealFeatures(fm_test_real)
	subkernel = GaussianKernel(10, width)
	feats_train.append_feature_obj(subkfeats_train)
	feats_test.append_feature_obj(subkfeats_test)
	kernel.append_kernel(subkernel)

	subkfeats_train = RealFeatures(fm_train_real)
	subkfeats_test = RealFeatures(fm_test_real)
	subkernel = LinearKernel()
	feats_train.append_feature_obj(subkfeats_train)
	feats_test.append_feature_obj(subkfeats_test)
	kernel.append_kernel(subkernel)

	subkfeats_train = RealFeatures(fm_train_real)
	subkfeats_test = RealFeatures(fm_test_real)
	subkernel = PolyKernel(10,2)
	feats_train.append_feature_obj(subkfeats_train)
	feats_test.append_feature_obj(subkfeats_test)
	kernel.append_kernel(subkernel)

	kernel.init(feats_train, feats_train)

	labels = MulticlassLabels(label_train_multiclass)

	mkl = MKLMulticlass(C, kernel, labels)

	mkl.set_epsilon(epsilon);
	mkl.parallel.set_num_threads(num_threads)
	mkl.set_mkl_epsilon(mkl_epsilon)
	mkl.set_mkl_norm(mkl_norm)

	mkl.train()

	kernel.init(feats_train, feats_test)

	out =  mkl.apply().get_labels()
	return out

if __name__ == '__main__':
	print('mkl_multiclass')
	mkl_multiclass_modular(*parameter_list[0])



#!/usr/bin/env python
from tools.multiclass_shared import prepare_data

[traindat, label_traindat, testdat, label_testdat] = prepare_data(False)

parameter_list = [[traindat,testdat,label_traindat,label_testdat,2.1,1,1e-5],[traindat,testdat,label_traindat,label_testdat,2.2,1,1e-5]]

def classifier_multiclass_shareboost (fm_train_real=traindat,fm_test_real=testdat,label_train_multiclass=label_traindat,label_test_multiclass=label_testdat,lawidth=2.1,C=1,epsilon=1e-5):
    from modshogun import RealFeatures, RealSubsetFeatures, MulticlassLabels
    from modshogun import ShareBoost

    #print('Working on a problem of %d features and %d samples' % fm_train_real.shape)

    feats_train = RealFeatures(fm_train_real)

    labels = MulticlassLabels(label_train_multiclass)

    shareboost = ShareBoost(feats_train, labels, min(fm_train_real.shape[0]-1, 30))
    shareboost.train();
    #print(shareboost.get_activeset())

    feats_test  = RealSubsetFeatures(RealFeatures(fm_test_real), shareboost.get_activeset())
    label_pred = shareboost.apply(feats_test)

    out = label_pred.get_labels()

    if label_test_multiclass is not None:
        from modshogun import MulticlassAccuracy
        labels_test = MulticlassLabels(label_test_multiclass)
        evaluator = MulticlassAccuracy()
        acc = evaluator.evaluate(label_pred, labels_test)
        #print('Accuracy = %.4f' % acc)

    return out

if __name__=='__main__':
    print('MulticlassMachine')
    classifier_multiclass_shareboost(*parameter_list[0])




#!/usr/bin/env python
#!/usr/bin/env python

import numpy as np

def gen_data(num_classes,num_samples,dim):
	np.random.seed(0)
	covs = np.array([[[0., -1. ], [2.5,  .7]],
			 [[3., -1.5], [1.2, .3]],
			 [[ 2,  0  ], [ .0,  1.5 ]]])
	X = np.r_[np.dot(np.random.randn(num_samples, dim), covs[0]) + np.array([0, 10]),
		  np.dot(np.random.randn(num_samples, dim), covs[1]) + np.array([-10, -10]),
		  np.dot(np.random.randn(num_samples, dim), covs[2]) + np.array([10, -10])];
	Y = np.hstack((np.zeros(num_samples), np.ones(num_samples), 2*np.ones(num_samples)))
	return X, Y

# Number of classes
M = 3
# Number of samples of each class
N = 50
# Dimension of the data
dim = 2

traindat, label_traindat = gen_data(M,N,dim)

parameter_list = [[traindat,label_traindat]]

def so_multiclass (fm_train_real=traindat,label_train_multiclass=label_traindat):
	try:
		from modshogun	import RealFeatures
		from modshogun	import MulticlassModel, MulticlassSOLabels, PrimalMosekSOSVM, RealNumber
	except ImportError:
		print("Mosek not available")
		return

	labels = MulticlassSOLabels(label_train_multiclass)
	features = RealFeatures(fm_train_real.T)

	model = MulticlassModel(features, labels)
	sosvm = PrimalMosekSOSVM(model, labels)
	sosvm.train()

	out = sosvm.apply()
	count = 0
	for i in xrange(out.get_num_labels()):
		yi_pred = RealNumber.obtain_from_generic(out.get_label(i))
		if yi_pred.value == label_train_multiclass[i]:
			count = count + 1

	print("Correct classification rate: %0.2f" % ( 100.0*count/out.get_num_labels() ))

if __name__=='__main__':
	print('SO multiclass')
	so_multiclass(*parameter_list[0])



#!/usr/bin/env python
from tools.load import LoadMatrix
lm=LoadMatrix()

traindna = lm.load_dna('../data/fm_train_dna.dat')
testdna = lm.load_dna('../data/fm_test_dna.dat')

parameter_list = [[traindna,testdna,3,0,False,False],[traindna,testdna,3,0,False,False]]

def preprocessor_sortwordstring_modular (fm_train_dna=traindna,fm_test_dna=testdna,order=3,gap=0,reverse=False,use_sign=False):

	from modshogun import CommWordStringKernel
	from modshogun import StringCharFeatures, StringWordFeatures, DNA
	from modshogun import SortWordString

	charfeat=StringCharFeatures(fm_train_dna, DNA)
	feats_train=StringWordFeatures(charfeat.get_alphabet())
	feats_train.obtain_from_char(charfeat, order-1, order, gap, reverse)
	preproc=SortWordString()
	preproc.init(feats_train)
	feats_train.add_preprocessor(preproc)
	feats_train.apply_preprocessor()

	charfeat=StringCharFeatures(fm_test_dna, DNA)
	feats_test=StringWordFeatures(charfeat.get_alphabet())
	feats_test.obtain_from_char(charfeat, order-1, order, gap, reverse)
	feats_test.add_preprocessor(preproc)
	feats_test.apply_preprocessor()

	kernel=CommWordStringKernel(feats_train, feats_train, use_sign)

	km_train=kernel.get_kernel_matrix()
	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()

	return km_train,km_test,kernel

if __name__=='__main__':
	print('CommWordString')
	preprocessor_sortwordstring_modular(*parameter_list[0])



#!/usr/bin/env python
from modshogun import RealFeatures, LongIntFeatures, ByteFeatures
from numpy import array, float64, int64, uint8, all

# create dense matrices A,B,C

matrixA=array([[1,2,3],[4,0,0],[0,0,0],[0,5,0],[0,0,6],[9,9,9]], dtype=float64)
matrixB=array([[1,2,3],[4,0,0],[0,0,0],[0,5,0],[0,0,6],[9,9,9]], dtype=int64)
matrixC=array([[1,2,3],[4,0,0],[0,0,0],[0,5,0],[0,0,6],[9,9,9]], dtype=uint8)

# ... of type Real, LongInt and Byte
parameter_list = [[matrixA,matrixB,matrixC]]

def features_dense_modular (A=matrixA,B=matrixB,C=matrixC):

    a=RealFeatures(A)
    b=LongIntFeatures(B)
    c=ByteFeatures(C)

# or 16bit wide ...
#feat1 = f.ShortFeatures(N.zeros((10,5),N.short))
#feat2 = f.WordFeatures(N.zeros((10,5),N.uint16))


# print(some statistics about a)

# get first feature vector and set it

    a.set_feature_vector(array([1,4,0,0,0,9], dtype=float64), 0)

# get matrices
    a_out = a.get_feature_matrix()
    b_out = b.get_feature_matrix()
    c_out = c.get_feature_matrix()

    assert(all(a_out==A))

    assert(all(b_out==B))

    assert(all(c_out==C))
    return a_out,b_out,c_out,a,b,c

if __name__=='__main__':
    print('dense')
    features_dense_modular(*parameter_list[0])



"""
Test Classifier
"""

from numpy import double
from modshogun import BinaryLabels
from modshogun import *
from modshogun import *
from modshogun import *

import util

def _get_machine (indata, prefix, feats):
	if indata[prefix+'type']=='kernel':
		pre='kernel_'
		kargs=util.get_args(indata, pre)
		kfun=eval(indata[pre+'name']+'Kernel')
		machine=kfun(feats['train'], feats['train'], *kargs)

		if indata[pre+'name']=='Linear':
			normalizer=eval(indata[pre+'normalizer']+'()')
			machine.set_normalizer(normalizer)
			machine.init(feats['train'], feats['train'])

		machine.parallel.set_num_threads(indata[prefix+'num_threads'])
	elif indata[prefix+'type']=='knn':
		pre='distance_'
		dargs=util.get_args(indata, pre)
		dfun=eval(indata[pre+'name'])
		machine=dfun(feats['train'], feats['train'], *dargs)
		machine.parallel.set_num_threads(indata[prefix+'num_threads'])
	else:
		machine=None

	return machine


def _get_results_alpha_and_sv(indata, prefix, classifier):
	if prefix+'alpha_sum' not in indata and \
		prefix+'sv_sum' not in indata:
		return None, None

	a=0
	sv=0
	if prefix+'label_type' in indata and \
		indata[prefix+'label_type']=='series':
		for i in range(classifier.get_num_svms()):
			subsvm=classifier.get_svm(i)
			for item in subsvm.get_alphas().tolist():
				a+=item
			for item in subsvm.get_support_vectors().tolist():
				sv+=item

		a=abs(a-indata[prefix+'alpha_sum'])
		sv=abs(sv-indata[prefix+'sv_sum'])
	else:
		for item in classifier.get_alphas().tolist():
			a+=item
		a=abs(a-indata[prefix+'alpha_sum'])
		for item in classifier.get_support_vectors().tolist():
			sv+=item
		sv=abs(sv-indata[prefix+'sv_sum'])

	return a, sv


def _get_results (indata, prefix, classifier, machine=None, feats=None):
	res={
		'alphas':0,
		'bias':0,
		'sv':0,
		'accuracy':indata[prefix+'accuracy'],
	}

	if prefix+'bias' in indata:
		res['bias']=abs(classifier.get_bias()-indata[prefix+'bias'])

	res['alphas'], res['sv']=_get_results_alpha_and_sv(
		indata, prefix, classifier)

	ctype=indata[prefix+'type']
	if ctype=='kernel' or ctype=='knn':
		machine.init(feats['train'], feats['test'])
	else:
		ctypes=('linear', 'perceptron', 'lda', 'wdsvmocas')
		if ctype in ctypes:
			classifier.set_features(feats['test'])

	res['classified']=max(abs(
		classifier.apply().get_values()-indata[prefix+'classified']))
	return res


def _evaluate (indata):
	prefix='classifier_'
	ctype=indata[prefix+'type']
	if indata[prefix+'name']=='KNN':
		feats=util.get_features(indata, 'distance_')
	elif ctype=='kernel':
		feats=util.get_features(indata, 'kernel_')
	else:
		feats=util.get_features(indata, prefix)

	machine=_get_machine(indata, prefix, feats)

	try:
		fun=eval(indata[prefix+'name'])
	except NameError as e:
		print("%s is disabled/unavailable!"%indata[prefix+'name'])
		return False

	# cannot refactor into function, because labels is unrefed otherwise
	if prefix+'labels' in indata:
		labels=BinaryLabels(double(indata[prefix+'labels']))
		if ctype=='kernel':
			classifier=fun(indata[prefix+'C'], machine, labels)
		elif ctype=='linear':
			classifier=fun(indata[prefix+'C'], feats['train'], labels)
		elif ctype=='knn':
			classifier=fun(indata[prefix+'k'], machine, labels)
		elif ctype=='lda':
			classifier=fun(indata[prefix+'gamma'], feats['train'], labels)
		elif ctype=='perceptron':
			classifier=fun(feats['train'], labels)
		elif ctype=='wdsvmocas':
			classifier=fun(indata[prefix+'C'], indata[prefix+'degree'],
				indata[prefix+'degree'], feats['train'], labels)
		else:
			return False
	else:
		classifier=fun(indata[prefix+'C'], machine)

	if classifier.get_name() == 'LibLinear':
		print(classifier.get_name(), "yes")
		classifier.set_liblinear_solver_type(L2R_LR)

	classifier.parallel.set_num_threads(indata[prefix+'num_threads'])
	if ctype=='linear':
		if prefix+'bias' in indata:
			classifier.set_bias_enabled(True)
		else:
			classifier.set_bias_enabled(False)
	if ctype=='perceptron':
		classifier.set_learn_rate=indata[prefix+'learn_rate']
		classifier.set_max_iter=indata[prefix+'max_iter']
	if prefix+'epsilon' in indata:
		try:
			classifier.set_epsilon(indata[prefix+'epsilon'])
		except AttributeError:
			pass
	if prefix+'max_train_time' in indata:
		classifier.set_max_train_time(indata[prefix+'max_train_time'])
	if prefix+'linadd_enabled' in indata:
		classifier.set_linadd_enabled(indata[prefix+'linadd_enabled'])
	if prefix+'batch_enabled' in indata:
		classifier.set_batch_computation_enabled(indata[prefix+'batch_enabled'])

	classifier.train()

	res=_get_results(indata, prefix, classifier, machine, feats)
	return util.check_accuracy(res['accuracy'],
		alphas=res['alphas'], bias=res['bias'], sv=res['sv'],
		classified=res['classified'])


########################################################################
# public
########################################################################

def test (indata):
	return _evaluate(indata)




import string

class predictions(object):
	def __init__(self, positions=None, scores=None):
		self.positions=positions
		self.scores=scores

	def set_positions(self, positions):
		self.positions=positions;
	def get_positions(self):
		return self.positions

	def set_scores(self, scores):
		self.scores=scores
	def get_scores(self):
		return self.scores

	def __str__(self):
		return 'positions: ' + `self.positions` + 'scores: ' + `self.scores`
	def __repr__(self):
		return self.__str__()

class sequence(object):
	def __init__(self, name, seq, (start,end)):
		assert(start<end<len(seq))
		self.start=start
		self.end=end
		self.name=name
		self.seq=seq
		self.preds=dict()
		self.preds['acceptor']=predictions()
		self.preds['donor']=predictions()

	def __str__(self):
		s="start:" + `self.start`
		s+=" end:" + `self.end`
		s+=" name:" + `self.name`
		s+=" sequence:" + `self.seq[0:10]`
		s+="... preds:" + `self.preds`
		return s
	def __repr__(self):
		return self.__str__()

def seqdict(dic, (start,end)):
	""" takes a fasta dict as input and
	generates a list of sequence objects from it """

	sequences=list()

	#translate string to ACGT / all non ACGT letters are mapped to A
	tab=''
	for i in xrange(256):
		if chr(i).upper() in 'ACGT':
			tab+=chr(i).upper()
		else:
			tab+='A'

	for seqname in dic.ordered_items():
		seq=string.translate(seqname[1], tab)
		seq=seq.upper()
		if end<0:
			stop=len(seq)+end
		else:
			stop=end

		sequences.append(sequence(seqname[0], seq, (start,stop)))

	return sequences



"""
This module contains code for generating toy examples
"""

#############################################################################################
#                                                                                           #
#    This program is free software; you can redistribute it and/or modify                   #
#    it under the terms of the GNU General Public License as published by                   #
#    the Free Software Foundation; either version 3 of the License, or                      #
#    (at your option) any later version.                                                    #
#                                                                                           #
#    This program is distributed in the hope that it will be useful,                        #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of                         #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                           #
#    GNU General Public License for more details.                                           #
#                                                                                           #
#    You should have received a copy of the GNU General Public License                      #
#    along with this program; if not, see http://www.gnu.org/licenses                       #
#    or write to the Free Software Foundation, Inc., 51 Franklin Street,                    #
#    Fifth Floor, Boston, MA 02110-1301  USA                                                #
#                                                                                           #
#############################################################################################

import sys
import parse

import random
from numpy.random import randn
from numpy import ones, concatenate, array, transpose
from esvm.mldata import DatasetFileFASTA, init_datasetfile
from esvm.mldata_arff import DatasetFileARFF

class MotifDataDef(object):
    motif = ''
    numseq = 0
    seqlenmin = 0
    seqlenmax = 0
    posstart = 0
    posend = 0
    mutrate = 0.0

################################################################################
# data generation functions

def motifgen(motif, numseq, seqlenmin, seqlenmax, posstart, posend, mutrate):
    """Generate sequences with a particular motif at a particular location.
    Also allow a possible mutation rate of the motif.
    """

    metadata = 'motifgen(%s,%d,%d,%d,%d,%d,%1.2f)' % (motif, numseq, seqlenmin, seqlenmax, posstart, posend, mutrate)

    acgt='acgt'
    seqlist = []
    for i in xrange(0,numseq):
        str=[] ;
        seqlen=random.randint(seqlenmin,seqlenmax) ;
        for l in xrange(0,seqlen):
            str.append(acgt[random.randint(0,3)])
        pos=random.randint(posstart,posend) ;
        for l in xrange(0,len(motif)):
            if (random.random()>=mutrate) and (pos+l<seqlen) and (pos+l>=0):
                str[pos+l]=motif[l]
        seqlist.append(''.join(str).upper())

    return metadata, seqlist


def cloudgen(numpoint, numfeat, fracpos, width):
    """Generate two Gaussian point clouds, centered around one and minus one."""

    numpos = int(round(fracpos*numpoint))
    numneg = numpoint - numpos

    metadata = 'cloudgen(%d,%d,%d,%3.2f)' % (numpos, numneg, numfeat, width)

    datapos = ones((numfeat, numpos)) + width*randn(numfeat, numpos)
    dataneg = -ones((numfeat, numneg)) + width*randn(numfeat, numneg)
    pointcloud = concatenate((datapos,dataneg),axis=1)
    labels = concatenate((ones(numpos),-ones(numneg)))

    return metadata, pointcloud, labels





################################################################################
# ARFF functions

def arffwrite_real(filename, numpoint, numfeat, fracpos=0.5, width=1.0):
    """Write an ARFF file containing a vectorial dataset"""
    #import arff

    (metadata, pointcloud, labels) = cloudgen(numpoint, numfeat, fracpos, width)

    fp = init_datasetfile(filename,'vec')
    fp.comment = metadata
    fp.dataname = 'pointcloud'
    fp.writelines(pointcloud,labels)


def arffwrite_sequence(filename,p, n):
    """Write an ARFF file containing a sequence dataset"""
    #import arff

    (metadatapos,seqlistpos) = motifgen(p.motif, p.numseq, p.seqlenmin, p.seqlenmax, p.posstart, p.posend, p.mutrate)
    (metadataneg,seqlistneg) = motifgen(n.motif, n.numseq, n.seqlenmin, n.seqlenmax, n.posstart, n.posend, n.mutrate)

    labels = concatenate((ones(len(seqlistpos)),-ones(len(seqlistneg))))
    seqlist = seqlistpos + seqlistneg
    fp = init_datasetfile(filename,'seq')
    fp.comment = metadatapos+' '+metadataneg
    fp.dataname = 'motif'
    fp.writelines(seqlist,labels)



def arffread(kernelname,datafilename):
    """Decide based on kernelname whether to read a sequence or vectorial file"""

    if kernelname == 'gauss' or kernelname == 'linear' or kernelname == 'poly' or kernelname == None:
        fp = init_datasetfile(datafilename,'vec')
    elif kernelname == 'wd' or kernelname == 'localalign' or kernelname == 'localimprove'\
             or kernelname == 'spec' or kernelname == 'cumspec':
        fp = init_datasetfile(datafilename,'seq')
    elif kernelname == 'spec2' or kernelname == 'cumspec2':
        fp = init_datasetfile(datafilename,'mseq')
    else:
        print 'Unknown kernel in arffread'

    return fp.readlines()

################################################################################
# fasta functions

def fastawrite_sequence(filename,p):
    """Write a FASTA file containing a sequence dataset"""
    import arff

    (metadata,seqlist) = motifgen(p.motif, p.numseq, p.seqlenmin, p.seqlenmax, p.posstart, p.posend, p.mutrate)
    labels = ones(len(seqlist))
    fp = init_datasetfile(filename,'seq')
    fp.writelines(seqlist,labels)

def fastaread(fnamepos,fnameneg=None):
    """Read two fasta files, the first positive, the second negative"""
    fpos = init_datasetfile(fnamepos,'seq')
    (fa1,lab1) = fpos.readlines()

    if fnameneg is not None:
        fneg = init_datasetfile(fnameneg,'seq')
        (fa2,lab2) = fneg.readlines()

        print 'positive: %d, negative %d' % (len(fa1),len(fa2))
        all_labels = concatenate((ones(len(fa1)),-ones(len(fa2))))
        all_examples = fa1 + fa2
    else:
        all_examples = fa1
        all_labels = ones(len(fa1))

    return all_examples, all_labels




"""
Blind Source Separation using the FFSep Algorithm with Shogun

Based on the example from scikit-learn
http://scikit-learn.org/

Kevin Hughes 2013
"""


import numpy as np
import pylab as pl

from modshogun  import RealFeatures
from modshogun import FFSep

# Generate sample data
np.random.seed(0)
n_samples = 2000
time = np.linspace(0, 10, n_samples)

# Source Signals
s1 = np.sin(2 * time)  # sin wave
s2 = np.sign(np.sin(3 * time))  # square wave
S = np.c_[s1, s2]
S += 0.2 * np.random.normal(size=S.shape)  # add noise

# Standardize data
S /= S.std(axis=0)
S = S.T

# Mixing Matrix
A = np.array([[1, 0.5], [0.5, 1]])

# Mix Signals
X = np.dot(A,S)
mixed_signals = RealFeatures(X)

# Separating
ffsep = FFSep()
signals = ffsep.apply(mixed_signals)
S_ = signals.get_feature_matrix()
A_ = ffsep.get_mixing_matrix();

# Plot results
pl.figure()
pl.subplot(3, 1, 1)
pl.plot(S.T)
pl.title('True Sources')
pl.subplot(3, 1, 2)
pl.plot(X.T)
pl.title('Mixed Sources')
pl.subplot(3, 1, 3)
pl.plot(S_.T)
pl.title('Estimated Sources')
pl.subplots_adjust(0.09, 0.04, 0.94, 0.94, 0.26, 0.36)
pl.show()



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Written (W) 2008 Soeren Sonnenburg
# Written (W) 2011 Christian Widmer
# Copyright (C) 2008-2011 Fraunhofer Institute FIRST and Max-Planck-Society

import numpy
import sys

from util import *

from shogun.Features import StringCharFeatures, StringWordFeatures, CombinedFeatures, DNA
from shogun.Kernel import CombinedKernel, WeightedDegreePositionStringKernel
from shogun.Kernel import K_COMMWORDSTRING, CommWordStringKernel, IdentityKernelNormalizer
from shogun.Preprocessor import SortWordString
from shogun.Classifier import KernelMachine


class Sensor(object):
    """
    Sensor has window (left,center,right) of length right-left+1
    with center at "center"
    """

    def __init__(self, window=None, kernel=None, train_features=None):
        self.kernel = kernel
        self.window = window
        self.train_features = train_features
        self.preproc = None

    def from_file(self, file, num):
        """
        parse lines with num as suffix, e.g.

        kernel_<arg><num>=<value>
        """
        l = file.readline()

        name = None
        left = None
        right = None
        center = None
        order = None
        shift = None
        svs = None

        while l:
            if l.find('%d=' % num) > -1:
                if name is None: name = parse_name(l, 'kernel_name%d' % num)
                if left is None: left = parse_int(l, 'kernel_left%d' % num)
                if right is None: right = parse_int(l, 'kernel_right%d' % num)
                if center is None: center = parse_int(l, 'kernel_center%d' % num)
                if order is None: order = parse_int(l, 'kernel_order%d' % num)
                if shift is None: shift = parse_int(l, 'kernel_shift%d' % num)
                if svs is None: svs = parse_string(l, file, 'kernel_svs%d' % num)
            else:
                self.window = (left, center, right)
                return self.init_sensor({ 'name' : name, 'order': order, 'shift' : shift}, svs)

            l = file.readline()

    def init_sensor(self, kernel, svs):
        f = StringCharFeatures(svs, DNA)

        kname = kernel['name']
        if  kname == 'spectrum':
            wf = StringWordFeatures(f.get_alphabet())
            wf.obtain_from_char(f, kernel['order'] - 1, kernel['order'], 0, False)

            pre = SortWordString()
            pre.init(wf)
            wf.add_preprocessor(pre)
            wf.apply_preprocessor()
            f = wf

            k = CommWordStringKernel(0, False)
            k.set_use_dict_diagonal_optimization(kernel['order'] < 8)
            self.preproc = pre

        elif kname == 'wdshift':
                k = WeightedDegreePositionStringKernel(0, kernel['order'])
                k.set_normalizer(IdentityKernelNormalizer())
                k.set_shifts(kernel['shift'] *
                        numpy.ones(f.get_max_vector_length(), dtype=numpy.int32))
                k.set_position_weights(1.0 / f.get_max_vector_length() *
                        numpy.ones(f.get_max_vector_length(), dtype=numpy.float64))
        else:
            raise "Currently, only wdshift and spectrum kernels supported"

        self.kernel = k
        self.train_features = f

        return (self.kernel, self.train_features)

    def get_test_features(self, seq, window):
        start = self.window[0] - window[0]
        end = len(seq) - window[1] + self.window[2]
        size = self.window[2] - self.window[0] + 1
        seq = seq[start:end]
        seq = seq.replace("N", "A").replace("R", "A").replace("M", "A")
        f = StringCharFeatures([seq], DNA)

        if self.preproc:
            wf = StringWordFeatures(f.get_alphabet())
            o = self.train_features.get_order()
            wf.obtain_from_char(f, 0, o, 0, False)
            f = wf
            f.obtain_by_sliding_window(size, 1, o - 1)
        else:
            f.obtain_by_sliding_window(size, 1)

        return f

class SignalSensor(object):
    """
    A collection of sensors
    """
    def __init__(self):
        self.sensors = list()
        self.kernel = CombinedKernel()
        self.svs = CombinedFeatures()
        self.svm = None
        self.window = (+100000, -1000000)

    def from_file(self, file):
        sys.stderr.write('loading model file')
        l = file.readline();

        if l != '%arts version: 1.0\n':
            sys.stderr.write("\nfile not an arts definition file\n")
            return None

        bias = None
        alphas = None
        num_kernels = None

        while l:
            # skip comment or empty line
            if not (l.startswith('%') or l.startswith('\n')):
                if bias is None: bias = parse_float(l, 'b')
                if alphas is None: alphas = parse_vector(l, file, 'alphas')
                if num_kernels is None: num_kernels = parse_int(l, 'num_kernels')

                if num_kernels and bias and alphas is not None:
                    for i in xrange(num_kernels):
                        s = Sensor()
                        (k, f) = s.from_file(file, i + 1)
                        k.io.enable_progress()
                        self.window = (min(self.window[0], s.window[0]),
                                max(self.window[1], s.window[2]))
                        self.sensors.append(s)
                        self.kernel.append_kernel(k)
                        self.svs.append_feature_obj(f)

                    self.kernel.init(self.svs, self.svs)
                    self.svm = KernelMachine(self.kernel, alphas,
                            numpy.arange(len(alphas), dtype=numpy.int32), bias)
                    self.svm.io.set_target_to_stderr()
                    self.svm.io.enable_progress()
                    self.svm.parallel.set_num_threads(self.svm.parallel.get_num_cpus())
                    sys.stderr.write('done\n')
                    return

            l = file.readline()

        sys.stderr.write('error loading model file\n')


    def predict(self, seq, chunk_size = int(10e6)):
        """
        predicts on whole contig, splits up sequence in chunks of size chunk_size
        """

        seq_len = len(seq)
        num_chunks = int(numpy.ceil(float(seq_len) / float(chunk_size)))
        assert(num_chunks > 0)

	sys.stderr.write("number of chunks for contig: %i\n" % (num_chunks))

        start = 0
        stop = min(chunk_size, seq_len)

        out = []

        # iterate over chunks
        for chunk_idx in range(num_chunks):

            sys.stderr.write("processing chunk #%i\n" % (chunk_idx))

            assert (start < stop)
            chunk = seq[start:stop]

            assert(len(self.sensors) > 0)
            tf = CombinedFeatures()
            for i in xrange(len(self.sensors)):
                f = self.sensors[i].get_test_features(chunk, self.window)
                tf.append_feature_obj(f)

            sys.stderr.write("initialising kernel...")
            self.kernel.init(self.svs, tf)
            sys.stderr.write("..done\n")

            self.svm.set_kernel(self.kernel)
            lab_out = self.svm.apply().get_values()

            assert(len(lab_out) > 0)
            out.extend(lab_out)

            # increment chunk
            start = stop
            stop = min(stop+chunk_size, seq_len)


        l = (-self.window[0]) * [-42]
        r = self.window[1] * [-42]

        # concatenate
        ret = l + out + r

        assert(len(ret) == len(seq))

        return ret



#!/usr/bin/env python
# vim: set fileencoding=utf-8

import sys

example_dir='../examples/documented'
target_dir='pages_cn/'

if len(sys.argv) > 1:
	sys.argv[1]
	target_dir=sys.argv[1]

directories={
		'python_modular' : ('Python模块化接口', 'ExamplesModularPython.mainpage',
		'''\n要运行这些例子只需要
\\verbatim
python name_of_example.py
\\endverbatim
'''
		),

		'r_modular' : ('R模块化接口', 'ExamplesModularR.mainpage',
		'''\n要运行这些例子只需要
\\verbatim
R -f name_of_example.R
\\endverbatim

或者启动R并输入
\\verbatim
source('name_of_example.R')
\\endverbatim
'''
		),

		'octave_modular': ('Octave模块化接口', 'ExamplesModularOctave.mainpage',
		'''\n要运行这些例子只需要
\\verbatim
octave name_of_example.m
\\endverbatim

或者启动octave并输入
\\verbatim
name_of_example
\\endverbatim
'''
		),

		'python_static': ('Python静态接口', 'ExamplesStaticPython.mainpage',
		'''\n要运行这些例子只需要
\\verbatim
python name_of_example.py
\\endverbatim
'''
		),

		'octave_static': ('Matlab(tm)和Octave静态接口', 'ExamplesStaticOctave.mainpage',
		'''\n要运行这些例子只需要
\\verbatim
octave name_of_example.m
\\endverbatim

或者启动octave或Matlab并输入
\\verbatim
name_of_example
\\endverbatim

注意，你要确保sg.oct或sg.mexglx（系统架构不同名字可能不同）已经在 matlab/octave可访问的路径中。
可通过下面的命令添加到它们的路径中：
\\verbatim
addpath /path/to/octave
\\endverbatim
以及
\\verbatim
addpath /path/to/matlab
\\endverbatim

最后请注意，如果是非root用户安装，你需要确保libshogun和libshogunui可以被动态链接器找到，你可能在启动matlab前需要设置：
\\verbatim
LD_LIBRARY_PATH=path/to/libshogun:path/to/libshogunui
\\endverbatim

		'''),

		'r_static' : ('R静态接口', 'ExamplesStaticR.mainpage',
		'''\n要运行这些例子只需要
\\verbatim
R -f name_of_example.R
\\endverbatim

或者启动R并输入
\\verbatim
source('name_of_example.R')
\\endverbatim
'''
		),

		'libshogun': ('C++ libshogun接口', 'ExamplesLibshogun.mainpage',
		'''\n如果要运行这些例子你需要像下面这样编译它们:
\\verbatim
g++ name_of_example.cpp -lshogun
\\endverbatim

如果你是将libshogun安装到非标准目录，你需要指定库目录和头文件目录，例如
\\verbatim
g++ -I/path/to/libshogun/includes name_of_example.cpp -L/path/to/libshogun/sofile -lshogun
\\endverbatim

如果shogun被安装到标准目录，可以像下面这样运行例子
\\verbatim
./name_of_example
\\endverbatim
相反如果被安装到非标准目录（那样它们不能被动态链接器找到），你需要指定
\\verbatim
LD_LIBRARY_PATH=path/to/libshogun ./name_of_example
\\endverbatim
		'''),

		'cmdline_static' :('命令行静态接口', 'ExamplesStaticCmdline.mainpage',
		'''\n要运行这些例子只需要
\\verbatim
shogun name_of_example.sg
\\endverbatim
		'''
			)}

valid_endings=['.py', '.m', '.R', '.sg', '.cpp']

import os
import os.path


for d in directories.keys():
	files=os.listdir(os.path.join(example_dir, d))
	files.sort()

	header=''
	body=''

	old_prefix=None

	header='/*! \page ' + d + '_examples ' + directories[d][0] + '例子\n\n'
	header+='本页面包含了所有' + directories[d][0] + '的例子。\n\n'
	for f in files:
		prefix=f[0:f.find('_')]
		suffix=f[f.rfind('.'):]
		#print prefix
		#print suffix
		if suffix not in valid_endings:
			continue
		if prefix != old_prefix:
			old_prefix=prefix
			body+='\n'
			header += '\\li \subpage ' + d + '_' + prefix + '_examples \n'
			body+='\\section ' + d + '_' + prefix + '_examples ' + prefix.title() + '\n\n'

		body += '\n\\li <b>' + os.path.join(example_dir, d, f) + '</b>\n'
		body += '\\verbinclude ' + f + '\n'

	header+=directories[d][2]
	text = header + body + '*/'
	file(os.path.join(target_dir, directories[d][1]),'w').write(text)



import modshogun as sg
import data
import numpy as np

# load data
feature_matrix = data.swissroll()
# create features instance
features = sg.RealFeatures(feature_matrix)

# create Multidimensional Scaling converter instance
converter = sg.MultidimensionalScaling()

# set target dimensionality
converter.set_target_dim(2)

# compute embedding with Multidimensional Scaling method
embedding = converter.embed(features)

# enable landmark approximation
converter.set_landmark(True)
# set number of landmarks
converter.set_landmark_number(100)
# set number of threads
converter.parallel.set_num_threads(2)
# compute approximate embedding
approx_embedding = converter.embed(features)
# disable landmark approximation
converter.set_landmark(False)

# compute cosine distance matrix 'manually'
N = features.get_num_vectors()
distance_matrix = np.zeros((N,N))
for i in range(N):
	for j in range(N):
		distance_matrix[i,j] = \
		  np.linalg.norm(feature_matrix[:,i]-feature_matrix[:,j],2)
# create custom distance instance
distance = sg.CustomDistance(distance_matrix)
# construct embedding based on created distance
converter.embed_distance(distance)



#!/usr/bin/env python
from tools.load import LoadMatrix
lm=LoadMatrix()
data=lm.load_cubes('../data/fm_train_cube.dat')

parameter_list=[[data, 1, 64, 1e-5, 2, 0, False, 5], [data, 3, 6, 1e-1, 1, 0, False, 2]]

def distribution_hmm_modular(fm_cube, N, M, pseudo, order, gap, reverse, num_examples):
	from modshogun import StringWordFeatures, StringCharFeatures, CUBE
	from modshogun import HMM, BW_NORMAL

	charfeat=StringCharFeatures(CUBE)
	charfeat.set_features(fm_cube)
	feats=StringWordFeatures(charfeat.get_alphabet())
	feats.obtain_from_char(charfeat, order-1, order, gap, reverse)

	hmm=HMM(feats, N, M, pseudo)
	hmm.train()
	hmm.baum_welch_viterbi_train(BW_NORMAL)

	num_examples=feats.get_num_vectors()
	num_param=hmm.get_num_model_parameters()
	for i in range(num_examples):
		for j in range(num_param):
			hmm.get_log_derivative(j, i)

	best_path=0
	best_path_state=0
	for i in range(num_examples):
		best_path+=hmm.best_path(i)
		for j in range(N):
			best_path_state+=hmm.get_best_path_state(i, j)

	lik_example = hmm.get_log_likelihood()
	lik_sample = hmm.get_log_likelihood_sample()

	return lik_example, lik_sample, hmm

###########################################################################
# call functions
###########################################################################

if __name__=='__main__':
	print('HMM')
	distribution_hmm_modular(*parameter_list[0])



"""
This module contains code for commonly used plots
"""

#############################################################################################
#                                                                                           #
#    This program is free software; you can redistribute it and/or modify                   #
#    it under the terms of the GNU General Public License as published by                   #
#    the Free Software Foundation; either version 3 of the License, or                      #
#    (at your option) any later version.                                                    #
#                                                                                           #
#    This program is distributed in the hope that it will be useful,                        #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of                         #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                           #
#    GNU General Public License for more details.                                           #
#                                                                                           #
#    You should have received a copy of the GNU General Public License                      #
#    along with this program; if not, see http://www.gnu.org/licenses                       #
#    or write to the Free Software Foundation, Inc., 51 Franklin Street,                    #
#    Fifth Floor, Boston, MA 02110-1301  USA                                                #
#                                                                                           #
#############################################################################################

import sys
import random
import numpy
import warnings
import shutil

from shogun.Features import Labels
from shogun.Evaluation import *

def plotroc(output, LTE, draw_random=False, figure_fname="", roc_label='ROC'):
    """Plot the receiver operating characteristic curve"""
    import pylab
    import matplotlib

    pylab.figure(1,dpi=150,figsize=(4,4))
    fontdict=dict(family="cursive",weight="bold",size=7,y=1.05) ;

    pm=PerformanceMeasures(Labels(numpy.array(LTE)), Labels(numpy.array(output)))

    points=pm.get_ROC()
    points=numpy.array(points).T # for pylab.plot
    pylab.plot(points[0], points[1], 'b-', label=roc_label)
    if draw_random:
        pylab.plot([0, 1], [0, 1], 'r-', label='random guessing')
    pylab.axis([0, 1, 0, 1])
    ticks=numpy.arange(0., 1., .1, dtype=numpy.float64)
    pylab.xticks(ticks,size=10)
    pylab.yticks(ticks,size=10)
    pylab.xlabel('1 - specificity (false positive rate)',size=10)
    pylab.ylabel('sensitivity (true positive rate)',size=10)
    pylab.legend(loc='lower right', prop = matplotlib.font_manager.FontProperties('tiny'))

    if figure_fname!=None:
        warnings.filterwarnings('ignore','Could not match*')
        tempfname = figure_fname + '.png'
	pylab.savefig(tempfname)
	shutil.move(tempfname,figure_fname)

    auROC=pm.get_auROC()
    return auROC ;

def plotprc(output, LTE, figure_fname="", prc_label='PRC'):
    """Plot the precision recall curve"""
    import pylab
    import matplotlib

    pylab.figure(2,dpi=150,figsize=(4,4))

    pm=PerformanceMeasures(Labels(numpy.array(LTE)), Labels(numpy.array(output)))

    points=pm.get_PRC()
    points=numpy.array(points).T # for pylab.plot
    pylab.plot(points[0], points[1], 'b-', label=prc_label)
    pylab.axis([0, 1, 0, 1])
    ticks=numpy.arange(0., 1., .1, dtype=numpy.float64)
    pylab.xticks(ticks,size=10)
    pylab.yticks(ticks,size=10)
    pylab.xlabel('sensitivity (true positive rate)',size=10)
    pylab.ylabel('precision (1 - false discovery rate)',size=10)
    pylab.legend(loc='lower right')

    if figure_fname!=None:
        warnings.filterwarnings('ignore','Could not match*')
        tempfname = figure_fname + '.png'
	pylab.savefig(tempfname)
	shutil.move(tempfname,figure_fname)

    auPRC=pm.get_auPRC()
    return auPRC ;

def plotcloud(cloud, figure_fname="", label='cloud'):
    """Plot the cloud of points (the first two dimensions only)"""
    import pylab
    import matplotlib

    pylab.figure(1,dpi=150,figsize=(4,4))

    pos = []
    neg = []
    for i in xrange(len(cloud)):
        if cloud[i][0]==1:
            pos.append(cloud[i][1:])
        elif cloud[i][0]==-1:
            neg.append(cloud[i][1:])

    fontdict=dict(family="cursive",weight="bold",size=10,y=1.05) ;
    pylab.title(label, fontdict)
    points=numpy.array(pos).T # for pylab.plot
    pylab.plot(points[0], points[1], 'b+', label='positive')
    points=numpy.array(neg).T # for pylab.plot
    pylab.plot(points[0], points[1], 'rx', label='negative')
    #pylab.axis([0, 1, 0, 1])
    #ticks=numpy.arange(0., 1., .1, dtype=numpy.float64)
    #pylab.xticks(ticks,size=10)
    #pylab.yticks(ticks,size=10)
    pylab.xlabel('dimension 1',size=10)
    pylab.ylabel('dimension 2',size=10)
    pylab.legend(loc='lower right')

    if figure_fname!=None:
        warnings.filterwarnings('ignore','Could not match*')
        tempfname = figure_fname + '.png'
	pylab.savefig(tempfname)
	shutil.move(tempfname,figure_fname)

def plot_poims(poimfilename, poim, max_poim, diff_poim, poim_totalmass, poimdegree, max_len):
    """Plot a summary of the information in poims"""
    import pylab
    import matplotlib

    pylab.figure(3, dpi=150, figsize=(4,5))

    # summary figures
    fontdict=dict(family="cursive",weight="bold",size=7,y=1.05) ;
    pylab.subplot(3,2,1)
    pylab.title('Total POIM Mass', fontdict)
    pylab.plot(poim_totalmass) ;
    pylab.ylabel('weight mass', size=5)

    pylab.subplot(3,2,3)
    pylab.title('POIMs', fontdict)
    pylab.pcolor(max_poim, shading='flat') ;

    pylab.subplot(3,2,5)
    pylab.title('Differential POIMs', fontdict)
    pylab.pcolor(diff_poim, shading='flat') ;

    for plot in [3, 5]:
        pylab.subplot(3,2,plot)
        ticks=numpy.arange(1., poimdegree+1, 1, dtype=numpy.float64)
        ticks_str = []
        for i in xrange(0, poimdegree):
            ticks_str.append("%i" % (i+1))
            ticks[i] = i + 0.5
        pylab.yticks(ticks, ticks_str)
        pylab.ylabel('degree', size=5)

    # per k-mer figures
    fontdict=dict(family="cursive",weight="bold",size=7,y=1.04) ;

    # 1-mers
    pylab.subplot(3,2,2)
    pylab.title('1-mer Positional Importance', fontdict)
    pylab.pcolor(poim[0], shading='flat') ;
    ticks_str = ['A', 'C', 'G', 'T']
    ticks = [0.5, 1.5, 2.5, 3.5]
    pylab.yticks(ticks, ticks_str, size=5)
    pylab.axis([0, max_len, 0, 4])

    # 2-mers
    pylab.subplot(3,2,4)
    pylab.title('2-mer Positional Importance', fontdict)
    pylab.pcolor(poim[1], shading='flat') ;
    i=0 ;
    ticks=[] ;
    ticks_str=[] ;
    for l1 in ['A', 'C', 'G', 'T']:
        for l2 in ['A', 'C', 'G', 'T']:
            ticks_str.append(l1+l2)
            ticks.append(0.5+i) ;
            i+=1 ;
    pylab.yticks(ticks, ticks_str, fontsize=5)
    pylab.axis([0, max_len, 0, 16])

    # 3-mers
    pylab.subplot(3,2,6)
    pylab.title('3-mer Positional Importance', fontdict)
    pylab.pcolor(poim[2], shading='flat') ;
    i=0 ;
    ticks=[] ;
    ticks_str=[] ;
    for l1 in ['A', 'C', 'G', 'T']:
        for l2 in ['A', 'C', 'G', 'T']:
            for l3 in ['A', 'C', 'G', 'T']:
                if numpy.mod(i,4)==0:
                    ticks_str.append(l1+l2+l3)
                    ticks.append(0.5+i) ;
                i+=1 ;
    pylab.yticks(ticks, ticks_str, fontsize=5)
    pylab.axis([0, max_len, 0, 64])

    # x-axis on last two figures
    for plot in [5, 6]:
        pylab.subplot(3,2,plot)
        pylab.xlabel('sequence position', size=5)


    # finishing up
    for plot in xrange(0,6):
        pylab.subplot(3,2,plot+1)
        pylab.xticks(fontsize=5)

    for plot in [1,3,5]:
        pylab.subplot(3,2,plot)
        pylab.yticks(fontsize=5)

    pylab.subplots_adjust(hspace=0.35) ;

    # write to file
    warnings.filterwarnings('ignore','Could not match*')
    pylab.savefig('/tmp/temppylabfig.png')
    shutil.move('/tmp/temppylabfig.png',poimfilename)




#!/usr/bin/env python

traindat = '../data/fm_train_real.dat'
testdat = '../data/fm_test_real.dat'
label_traindat = '../data/label_train_multiclass.dat'

parameter_list = [[traindat,testdat,label_traindat,3]]

def classifier_knn_modular(train_fname=traindat,test_fname=testdat,label_train_fname=label_traindat, k=3):
	from modshogun import RealFeatures, MulticlassLabels, KNN, EuclideanDistance, CSVFile

	feats_train=RealFeatures(CSVFile(train_fname))
	feats_test=RealFeatures(CSVFile(test_fname))
	distance=EuclideanDistance(feats_train, feats_train)

	labels=MulticlassLabels(CSVFile(label_train_fname))

	knn=KNN(k, distance, labels)
	knn_train = knn.train()
	output=knn.apply(feats_test).get_labels()
	multiple_k=knn.classify_for_multiple_k()

	return knn,knn_train,output,multiple_k

if __name__=='__main__':
	print('KNN')
	classifier_knn_modular(*parameter_list[0])



from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()

traindat=lm.load_numbers('../data/fm_train_real.dat')
testdat=lm.load_numbers('../data/fm_test_real.dat')
trainlabel=lm.load_labels('../data/label_train_multiclass.dat')
parameter_list=[[traindat,testdat,trainlabel,10,1.2,1.2,1e-5,0.001,1.5,1.0],
		[traindat,testdat,trainlabel,11,1.3,1.3,1e-5,0.002,1.6,1.1]]

def mkl_multiclass (fm_train_real=traindat,fm_test_real=testdat,
		label_train_multiclass=trainlabel,
		size_cache=10,width=1.2,C=1.2,epsilon=1e-5,
		mkl_eps=0.001,mkl_norm=1.5,weight=1.0):

	sg('clean_kernel')
	sg('clean_features', 'TRAIN')
	sg('clean_features', 'TEST')
	sg('set_kernel', 'COMBINED', size_cache)
	sg('add_kernel', weight, 'LINEAR', 'REAL', size_cache)
	sg('add_features', 'TRAIN', fm_train_real)
	sg('add_features', 'TEST', fm_test_real)
	sg('add_kernel', weight, 'GAUSSIAN', 'REAL', size_cache, width)
	sg('add_features', 'TRAIN', fm_train_real)
	sg('add_features', 'TEST', fm_test_real)
	sg('add_kernel', weight, 'POLY', 'REAL', size_cache, 2)
	sg('add_features', 'TRAIN', fm_train_real)
	sg('add_features', 'TEST', fm_test_real)

	sg('set_labels', 'TRAIN', label_train_multiclass)
	sg('new_classifier', 'MKL_MULTICLASS')
	sg('svm_epsilon', epsilon)
	sg('c', C)
	sg('mkl_parameters', mkl_eps, 0.0, mkl_norm)
	sg('train_classifier')

	#sg('set_features', 'TEST', fm_test_real)
	result=sg('classify')
	return result

if __name__=='__main__':
	print('mkl_multiclass')
	mkl_multiclass(*parameter_list[0])



#!/usr/bin/env python
parameter_list = [['features_string_char_compressed_modular.py']]

def features_string_char_compressed_modular (fname):
	from modshogun import StringCharFeatures, StringFileCharFeatures, RAWBYTE
	from modshogun import UNCOMPRESSED,SNAPPY,LZO,GZIP,BZIP2,LZMA, MSG_DEBUG
	from modshogun import DecompressCharString

	f=StringFileCharFeatures(fname, RAWBYTE)

	#print("original strings", f.get_features())

	#uncompressed
	f.save_compressed("foo_uncompressed.str", UNCOMPRESSED, 1)
	f2=StringCharFeatures(RAWBYTE);
	f2.load_compressed("foo_uncompressed.str", True)
	#print("uncompressed strings", f2.get_features())
	#print

	# load compressed data and uncompress on load

	#snappy - not stable yet?!
	#f.save_compressed("foo_snappy.str", SNAPPY, 9)
	#f2=StringCharFeatures(RAWBYTE);
	#f2.load_compressed("foo_snappy.str", True)
	#print("snappy strings", f2.get_features())
	#print

	#lzo
	f.save_compressed("foo_lzo.str", LZO, 9)
	f2=StringCharFeatures(RAWBYTE);
	f2.load_compressed("foo_lzo.str", True)
	#print("lzo strings", f2.get_features())
	#print

	##gzip
	f.save_compressed("foo_gzip.str", GZIP, 9)
	f2=StringCharFeatures(RAWBYTE);
	f2.load_compressed("foo_gzip.str", True)
	#print("gzip strings", f2.get_features())
	#print

	#bzip2
	f.save_compressed("foo_bzip2.str", BZIP2, 9)
	f2=StringCharFeatures(RAWBYTE);
	f2.load_compressed("foo_bzip2.str", True)
	#print("bzip2 strings", f2.get_features())
	#print

	#lzma
	f.save_compressed("foo_lzma.str", LZMA, 9)
	f2=StringCharFeatures(RAWBYTE);
	f2.load_compressed("foo_lzma.str", True)
	#print("lzma strings", f2.get_features())
	#print

	# load compressed data and uncompress via preprocessor
	f2=StringCharFeatures(RAWBYTE);
	f2.load_compressed("foo_lzo.str", False)
	f2.add_preprocessor(DecompressCharString(LZO))
	f2.apply_preprocessor()
	#print("lzo strings", f2.get_features())
	#print

	# load compressed data and uncompress on-the-fly via preprocessor
	f2=StringCharFeatures(RAWBYTE);
	f2.load_compressed("foo_lzo.str", False)
	#f2.io.set_loglevel(MSG_DEBUG)
	f2.add_preprocessor(DecompressCharString(LZO))
	f2.enable_on_the_fly_preprocessing()
	#print("lzo strings", f2.get_features())
	#print

	#clean up
	import os
	for f in ['foo_uncompressed.str', 'foo_snappy.str', 'foo_lzo.str', 'foo_gzip.str',
	'foo_bzip2.str', 'foo_lzma.str', 'foo_lzo.str', 'foo_lzo.str']:
		if os.path.exists(f):
			os.unlink(f)

	##########################################################################################
	# some perfectly compressible stuff follows
	##########################################################################################
	##########################################################################################
	##########################################################################################
	##########################################################################################
	##########################################################################################
	##########################################################################################
	##########################################################################################
	##########################################################################################
	##########################################################################################
	##########################################################################################

if __name__=='__main__':
    print('Compressing StringCharFileFeatures')
    features_string_char_compressed_modular(*parameter_list[0])



#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Written (W) 2006-2008 Soeren Sonnenburg
# Written (W) 2007 Gunnar Raetsch
# Copyright (C) 2006-2008 Fraunhofer Institute FIRST and Max-Planck-Society
#

import sys
import numpy
import seqdict

from modshogun import KernelMachine,StringCharFeatures,DNA,WeightedDegreeStringKernel

class svm_splice_model(object):
	def __init__(self, order, traindat, alphas, b, (window_left,offset,window_right), consensus):

		f=StringCharFeatures(DNA)
		f.set_features(traindat)
		wd_kernel = WeightedDegreeStringKernel(f,f, int(order))
		wd_kernel.io.set_target_to_stderr()

		self.svm=KernelMachine(wd_kernel, alphas, numpy.arange(len(alphas), dtype=numpy.int32), b)
		self.svm.io.set_target_to_stderr()
		self.svm.parallel.set_num_threads(self.svm.parallel.get_num_cpus())
		self.svm.set_linadd_enabled(False)
		self.svm.set_batch_computation_enabled(False)

		self.window_left=int(window_left)
		self.window_right=int(window_right)

		self.consensus=consensus
		self.wd_kernel=wd_kernel
		self.traindat=f
		self.offset=offset

	def get_positions(self, sequence):
		positions=list()

		for cons in self.consensus:
			l=sequence.find(cons)
			while l>-1:
				if l<len(sequence)-self.window_right-2 and l>self.window_left:
					positions.append(l+self.offset)
				l=sequence.find(cons, l+1)

		positions.sort()
		return positions

	def get_predictions_from_seqdict(self, seqdic, site):
		""" we need to generate a huge test features object
			containing all locations found in each seqdict-sequence
			and each location (this is necessary to efficiently
			(==fast,low memory) compute the splice outputs
		"""

		seqlen=self.window_right+self.window_left+2

		num=0
		for s in seqdic:
			num+= len(s.preds[site].positions)

		testdat = []

		for s in seqdic:
			sequence=s.seq
			positions=s.preds[site].positions
			for j in xrange(len(positions)):
				i=positions[j] - self.offset
				s=sequence[i-self.window_left:i+self.window_right+2]
				testdat.append(s)

		t=StringCharFeatures(testdat, DNA)

		self.wd_kernel.init(self.traindat, t)
		self.svm.set_kernel(self.wd_kernel)
		l=self.svm.apply().get_labels()
		sys.stderr.write("\n...done...\n")

		k=0
		for s in seqdic:
			num=len(s.preds[site].positions)
			scores= num * [0]
			for j in xrange(num):
				scores[j]=l[k]
				k+=1
			s.preds[site].set_scores(scores)

	def get_positions_from_seqdict(self, seqdic, site):
		for d in seqdic:
			positions=list()
			sequence=d.seq
			for cons in self.consensus:
				l=sequence.find(cons)
				while l>-1:
					if l<len(sequence)-self.window_right-2 and l>self.window_left:
						positions.append(l+self.offset)
					l=sequence.find(cons, l+1)
			positions.sort()
			d.preds[site].set_positions(positions)

	def get_predictions(self, sequence, positions):

		seqlen=self.window_right+self.window_left+2
		num=len(positions)

		testdat = []

		for j in xrange(num):
			i=positions[j] - self.offset ;
			s=sequence[i-self.window_left:i+self.window_right+2]
			testdat.append(s)

		t=StringCharFeatures(DNA)
		t.set_string_features(testdat)

		self.wd_kernel.init(self.traindat, t)
		l=self.svm.classify().get_labels()
		sys.stderr.write("\n...done...\n")
		return l

class signal_detectors(object):
	def __init__(self, model):
		if model.don_splice_use_gc:
			don_consensus=['GC','GT']
		else:
			don_consensus=['GT']

		self.acceptor=svm_splice_model(model.acc_splice_order, model.acc_splice_svs,
				numpy.array(model.acc_splice_alphas).flatten(), model.acc_splice_b,
				(model.acc_splice_window_left, 2, model.acc_splice_window_right), ['AG'])
		self.donor=svm_splice_model(model.don_splice_order, model.don_splice_svs,
				numpy.array(model.don_splice_alphas).flatten(), model.don_splice_b,
				(model.don_splice_window_left, 0, model.don_splice_window_right),
				don_consensus)

	def set_sequence(self, seq):
		self.acceptor.set_sequence(seq)
		self.donor.set_sequence(seq)

	def predict_acceptor_sites(self, seq):
		pos=self.acceptor.get_positions(seq)
		sys.stderr.write("computing svm output for acceptor positions\n")
		pred=self.acceptor.get_predictions(seq, pos)
		return (pos,pred)

	def predict_donor_sites(self,seq):
		pos=self.donor.get_positions(seq)
		sys.stderr.write("computing svm output for donor positions\n")
		pred=self.donor.get_predictions(seq, pos)
		return (pos,pred)

	def predict_acceptor_sites_from_seqdict(self, seqs):
		self.acceptor.get_positions_from_seqdict(seqs, 'acceptor')
		sys.stderr.write("computing svm output for acceptor positions\n")
		self.acceptor.get_predictions_from_seqdict(seqs, 'acceptor')

	def predict_donor_sites_from_seqdict(self, seqs):
		self.donor.get_positions_from_seqdict(seqs, 'donor')
		sys.stderr.write("computing svm output for donor positions\n")
		self.donor.get_predictions_from_seqdict(seqs, 'donor')



#!/usr/bin/env python
strings=['hey','guys','i','am','a','string']

parameter_list=[[strings]]

def features_hasheddocdot_modular(strings):
	from modshogun import StringCharFeatures, RAWBYTE
	from modshogun import HashedDocDotFeatures
	from modshogun import NGramTokenizer
	from numpy import array

	#create string features
	f=StringCharFeatures(strings, RAWBYTE)

	#set the number of bits of the target dimension
	#means a dim of size 2^5=32
	num_bits=5

	#create the ngram tokenizer of size 8 to parse the strings
	tokenizer=NGramTokenizer(8)

	#normalize results
	normalize=True

	#create HashedDocDot features
	hddf=HashedDocDotFeatures(num_bits, f, tokenizer, normalize)

	#should expect 32
	#print('Feature space dimensionality is', hddf.get_dim_feature_space())

	#print('Self dot product of string 0', hddf.dot(0, hddf, 0))

	return hddf

if __name__=='__main__':
	print('HashedDocDotFeatures')
	features_hasheddocdot_modular(*parameter_list[0])



from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()

traindna=lm.load_dna('../data/fm_train_dna.dat')
testdna=lm.load_dna('../data/fm_test_dna.dat')
trainlabel=lm.load_labels('../data/label_train_dna.dat')
parameter_list=[[traindna,testdna,trainlabel,10,3,0,'n',False,'FULL'],
		[traindna,testdna,trainlabel,11,4,0,'n',False,'FULL']]

def kernel_weightedcommwordstring (fm_train_dna=traindna,fm_test_dna=testdna,
				   label_train_dna=trainlabel,size_cache=10,
				   order=3,gap=0,reverse='n',use_sign=False,
				   normalization='FULL'):

	sg('add_preproc', 'SORTWORDSTRING')
	sg('set_features', 'TRAIN', fm_train_dna, 'DNA')
	sg('convert', 'TRAIN', 'STRING', 'CHAR', 'STRING', 'WORD', order, order-1, gap, reverse)
	sg('attach_preproc', 'TRAIN')

	sg('set_features', 'TEST', fm_test_dna, 'DNA')
	sg('convert', 'TEST', 'STRING', 'CHAR', 'STRING', 'WORD', order, order-1, gap, reverse)
	sg('attach_preproc', 'TEST')

	sg('set_kernel', 'WEIGHTEDCOMMSTRING', 'WORD', size_cache, use_sign, normalization)
	km=sg('get_kernel_matrix', 'TRAIN')
	km=sg('get_kernel_matrix', 'TEST')
	return km

if __name__=='__main__':
	print('WeightedCommWordString')
	kernel_weightedcommwordstring(*parameter_list[0])



#!/usr/bin/python

import os

opts=''
#opts=' --prefix=/local/cong --libs=/local/cong/lib --includes=/local/cong/include'

interfaces = ['elwms','cmdline', 'python_modular', 'python', 'r', 'r_modular',
              'octave', 'octave_modular', 'matlab','libshogun','libshogunui'];

def powerset(s):
    """Return the powerset of a list"""
    d = dict(zip(
        (1<<i for i in range(len(s))),
        (set([e]) for e in s)))

    subset = set()
    yield subset
    for i in range(1, 1<<len(s)):
        subset = subset ^ d[i & -i]
        yield subset


def test_configure():
    """Main"""
    exit_status = []
    for conf in powerset(interfaces):
        conf=tuple(conf)
        interface_string = ','.join(conf)
        p = os.system('./configure --interfaces=' + interface_string + opts + ' >/dev/null 2>&1')
        exit_status.append((conf,p))
        print conf,p
    print exit_status

if __name__ == '__main__':
    test_configure()



import modshogun as sg
import data
import numpy as np

# load data
feature_matrix = data.swissroll()
# create features instance
features = sg.RealFeatures(feature_matrix)

# create Kernel Locally Linear Embedding converter instance
converter = sg.KernelLocallyLinearEmbedding()

# set target dimensionality
converter.set_target_dim(2)
# set number of neighbors
converter.set_k(10)
# set number of threads
converter.parallel.set_num_threads(2)
# set nullspace shift (optional)
converter.set_nullspace_shift(-1e-6)

# create Gaussian kernel instance
kernel = sg.GaussianKernel(100,10.0)
# enable converter instance to use created kernel instance
converter.set_kernel(kernel)

# compute embedding with Kernel Locally Linear Embedding method
embedding = converter.embed(features)

# compute linear kernel matrix
kernel_matrix = np.dot(feature_matrix.T,feature_matrix)
# create Custom Kernel instance
custom_kernel = sg.CustomKernel(kernel_matrix)
# construct embedding based on created kernel
kernel_embedding = converter.embed_kernel(custom_kernel)



from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()

traindat=lm.load_numbers('../data/fm_train_real.dat')
parameter_list=[[traindat,10,3,1000],[traindat,11,4,1500]]

def clustering_kmeans (fm_train=traindat, size_cache=10,k=3,iter=1000):
	sg('set_features', 'TRAIN', fm_train)
	sg('set_distance', 'EUCLIDEAN', 'REAL')
	sg('new_clustering', 'KMEANS')
	sg('train_clustering', k, iter)

	[radi, centers]=sg('get_clustering')
	return [radi, centers]

if __name__=='__main__':
	print('KMeans')
	clustering_kmeans(*parameter_list[0])



from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()


traindat=lm.load_numbers('../data/fm_train_real.dat')
testdat=lm.load_numbers('../data/fm_test_real.dat')
train_label=lm.load_labels('../data/label_train_multiclass.dat')
parameter_list=[[traindat,testdat, train_label,3],
		[traindat,testdat,train_label,4]]

def classifier_knn (fm_train_real=traindat,fm_test_real=testdat,
			label_train_multiclass=train_label,k=3):

	sg('set_features', 'TRAIN', fm_train_real)
	sg('set_labels', 'TRAIN', label_train_multiclass)
	sg('set_distance', 'EUCLIDEAN', 'REAL')
	sg('new_classifier', 'KNN')
	sg('train_classifier', k)

	sg('set_features', 'TEST', fm_test_real)
	result=sg('classify')
	return result

if __name__=='__main__':
	print('KNN')
	classifier_knn(*parameter_list[0])



#!/usr/bin/env python
import sys

example_dir='../examples/documented'
target_dir='pages/'

if len(sys.argv) > 1:
	sys.argv[1]
	target_dir=sys.argv[1]

directories={
		'python_modular' : ('Python Modular', 'ExamplesModularPython.mainpage',
		'''\nTo run the examples issue
\\verbatim
python name_of_example.py
\\endverbatim
'''
		),

		'r_modular' : ('R Modular', 'ExamplesModularR.mainpage',
		'''\nTo run the examples issue
\\verbatim
R -f name_of_example.R
\\endverbatim

or start R and then type
\\verbatim
source('name_of_example.R')
\\endverbatim
'''
		),

		'octave_modular': ('Octave Modular', 'ExamplesModularOctave.mainpage',
		'''\nTo run the examples issue
\\verbatim
octave name_of_example.m
\\endverbatim

or start up octave and then type
\\verbatim
name_of_example
\\endverbatim
'''
		),

		'python_static': ('Static Python', 'ExamplesStaticPython.mainpage',
		'''\nTo run the examples issue
\\verbatim
python name_of_example.py
\\endverbatim
'''
		),

		'octave_static': ('Static Matlab(tm) and Octave', 'ExamplesStaticOctave.mainpage',
		'''\nTo run the examples issue
\\verbatim
octave name_of_example.m
\\endverbatim

or start up matlab or octave and then type
\\verbatim
name_of_example
\\endverbatim

Note that you have to make sure that the sg.oct or sg.mexglx (name varies with architecture)
has to be in the matlab/octave path. This can be achieved using the addpath command:
\\verbatim
addpath /path/to/octave
\\endverbatim
respectively
\\verbatim
addpath /path/to/matlab
\\endverbatim

Finally note that for non-root installations you will have to make sure that libshogun and libshogun ui can be found by the dynamic linker, e.g. you will need to set:

\\verbatim
LD_LIBRARY_PATH=path/to/libshogun:path/to/libshogunui
\\endverbatim
before starting matlab.
		'''),

		'r_static' : ('Static R', 'ExamplesStaticR.mainpage',
		'''\nTo run the examples issue
\\verbatim
R -f name_of_example.R
\\endverbatim

or start R and then type
\\verbatim
source('name_of_example.R')
\\endverbatim
'''
		),

		'libshogun': ('C++ libshogun', 'ExamplesLibshogun.mainpage',
		'''\nTo run the examples you will need to manually compile them via
\\verbatim
g++ name_of_example.cpp -lshogun
\\endverbatim

in case you installed libshogun to a nonstandard directory you will need to specify the appropriate library and include paths, e.g.
\\verbatim
g++ -I/path/to/libshogun/includes name_of_example.cpp -L/path/to/libshogun/sofile -lshogun
\\endverbatim

Then the examples are standard binary executables and can be started via
\\verbatim
./name_of_example
\\endverbatim
respectively if the libraries are in nonstandard locations (such that they cannot be found by the dynamic linker)
\\verbatim
LD_LIBRARY_PATH=path/to/libshogun ./name_of_example
\\endverbatim
		'''),

		'cmdline_static' :('Static Command Line', 'ExamplesStaticCmdline.mainpage',
		'''\nTo run the examples issue
\\verbatim
shogun name_of_example.sg
\\endverbatim
		'''
		),

		'lua_modular' : ('Lua Modular', 'ExamplesModularLua.mainpage',
		'''\nTo run the examples issue
\\verbatim
lua name_of_example.lua
\\endverbatim
'''
		),

		'ruby_modular' : ('Ruby Modular', 'ExamplesModularRuby.mainpage',
		'''\nTo run the examples issue
\\verbatim
ruby name_of_example.rb
\\endverbatim
'''
		),

		'csharp_modular' : ('C# Modular', 'ExamplesModularCSharp.mainpage',
		'''\nTo run the examples issue
\\verbatim
gmcs path/to/shogun/interfaces/csharp_modular/*.cs name_of_example.cs
LD_LIBRARY_PATH=path/to/libshogun:path/to/shogun/interfaces/csharp_modular mono name_of_example.exe
\\endverbatim
'''
		),

		'java_modular' : ('Java Modular', 'ExamplesModularJava.mainpage',
		'''\nTo run the examples issue
\\verbatim
javac -jar path/to/modshogun.jar name_of_example.java
java -jar path/to/modshogun.jar name_of_example
\\endverbatim
'''
		),
		}

valid_endings=['.py', '.m', '.R', '.sg', '.cpp', '.lua', '.rb', '.cs', '.java']

import os
import os.path


for d in directories.keys():
	files=os.listdir(os.path.join(example_dir, d))
	files.sort()

	header=''
	body=''

	old_prefix=None

	header='/*! \page ' + d + '_examples Examples for ' + directories[d][0] + ' Interface\n\n'
	header+='This page lists ready to run shogun examples for the ' + directories[d][0] + ' interface.\n\n'
	for f in files:
		if f.find('_') == -1:
			continue

		prefix=f[0:f.find('_')]
		suffix=f[f.rfind('.'):]
		#print prefix
		#print suffix
		if suffix not in valid_endings:
			continue
		if prefix != old_prefix:
			old_prefix=prefix
			body+='\n'
			header += '\\li \subpage ' + d + '_' + prefix + '_examples \n'
			body+='\\section ' + d + '_' + prefix + '_examples ' + prefix.title() + '\n\n'

		body += '\n\\li <b>' + os.path.join(example_dir, d, f) + '</b>\n'
		body += '\\verbinclude ' + f + '\n'

	header+=directories[d][2]
	text = header + body + '*/'
	file(os.path.join(target_dir, directories[d][1]),'w').write(text)



from elwms import elwms
import numpy
x=numpy.array([[1,2,3],[4,5,6]],dtype=numpy.float64)
y=numpy.array([[7,8,9],[0,1,2]],dtype=numpy.float64)

#elwms('loglevel', 'ALL')
#elwms('run_octave','octavecode', 'disp("hi")')
a,b,c=elwms('run_octave','x', x, 'y', y,
		'octavecode', 'class(x), disp(x), results=list(x+y,1,{"a"})')
res1=elwms('run_octave','x', x, 'y', y,
		'octavecode', 'disp(x); disp(y); results=x+y+rand(2,3)\n')
res2=elwms('run_octave','A', ['test','bla','foo'],
		'octavecode',
'''
disp(A);
disp("hi");
results={"a","b","c"}
''')



#!/usr/bin/env python
traindat = '../data/fm_train_real.dat'
testdat = '../data/fm_test_real.dat'
label_traindat = '../data/label_train_twoclass.dat'

parameter_list = [[traindat,testdat,label_traindat,0.9,1,6],[traindat,testdat,label_traindat,0.8,1,5]]

def classifier_svmsgd_modular (train_fname=traindat,test_fname=testdat,label_fname=label_traindat,C=0.9,num_threads=1,num_iter=5):
	from modshogun import RealFeatures, SparseRealFeatures, BinaryLabels
	from modshogun import SVMSGD, CSVFile

	feats_train=RealFeatures(CSVFile(train_fname))
	feats_test=RealFeatures(CSVFile(test_fname))
	labels=BinaryLabels(CSVFile(label_fname))

	svm=SVMSGD(C, feats_train, labels)
	svm.set_epochs(num_iter)
	#svm.io.set_loglevel(0)
	svm.train()

	bias=svm.get_bias()
	w=svm.get_w()
	predictions = svm.apply(feats_test)
	return predictions, svm, predictions.get_labels()

if __name__=='__main__':
	print('SVMSGD')
	classifier_svmsgd_modular(*parameter_list[0])



#!/usr/bin/env python

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Written (W) 2011 Sergey Lisitsyn
# Copyright (C) 2011 Sergey Lisitsyn

from modshogun import *
from numpy import *
from matplotlib.offsetbox import TextArea, DrawingArea, OffsetImage, AnnotationBbox
import re,os,time
from pylab import *

def build_features(path):
	files = os.listdir(path)
	files.remove('README')
	N = len(files)
	(nd,md) = imread(os.path.join(path,files[0])).shape
	dim = nd*md
	feature_matrix = zeros([dim,N])
	for i,filename in enumerate(files):
		feature_matrix[:,i] = imread(os.path.join(path,filename)).ravel()
	return nd,md,RealFeatures(feature_matrix)

path = '../../data/faces/'
converter = DiffusionMaps
nd,md,features = build_features(path)
converter_instance = converter()
converter_instance.set_t(5)
converter_instance.set_target_dim(2)

start = time.time()
new_features = converter_instance.embed(features).get_feature_matrix()
print new_features.shape
end = time.time()

clusterer = KMeans
clusterer_instance = clusterer(2,EuclideanDistance())
clusterer_instance.train(features)
labels = clusterer_instance.apply().get_labels()
print labels

print 'applied %s, took %fs' % (converter_instance.get_name(), end-start)
print 'plotting'

fig = figure()
ax = fig.add_subplot(111,axisbg='#ffffff')
ax.scatter(new_features[0],new_features[1],color='black')
import random
for i in range(len(new_features[0])):
	feature_vector = features.get_feature_vector(i)
	Z = zeros([nd,md,4])
	Z[:,:,0] = 255-feature_vector.reshape(nd,md)[::-1,:]
	Z[:,:,1] = Z[:,:,0]
	Z[:,:,2] = Z[:,:,0]
	for k in range(nd):
		for j in range(md):
			Z[k,j,3] = pow(sin(k*pi/nd)*sin(j*pi/md),0.5)
	imagebox = OffsetImage(Z,cmap=cm.gray,zoom=0.25)
	ab = AnnotationBbox(imagebox, (new_features[0,i],new_features[1,i]),
						pad=0.001,frameon=False)
	ax.add_artist(ab)
axis('off')
savefig('faces.png')
show()



#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Written (W) 2006-2007 Soeren Sonnenburg
# Written (W) 2007 Gunnar Raetsch
# Copyright (C) 2007-2008 Fraunhofer Institute FIRST and Max-Planck-Society
#

from numpy import array
from shogun.Structure import Plif
from shogun.Structure import PlifArray
from shogun.Library import DynamicPlifArray

class plif:
	def __init__(self, model):
		min_exon_len=2
		min_intron_len=30
		max_len=22222;
		#extract plifs from model
		l=array(model.penalty_acceptor_boundaries).flatten()
		p=array(model.penalty_acceptor_penalty).flatten()
		self.acceptor=Plif(len(l))
		self.acceptor.set_plif_limits(l)
		self.acceptor.set_plif_penalty(p)
		self.acceptor.set_min_value(-1e+20)
		self.acceptor.set_max_value(1e+20)
		self.acceptor.set_plif_name("acceptor")

		l=array(model.penalty_donor_boundaries).flatten()
		p=array(model.penalty_donor_penalty).flatten()
		self.donor=Plif(len(l))
		self.donor.set_plif_limits(l)
		self.donor.set_plif_penalty(p)
		self.donor.set_min_value(-1e+20)
		self.donor.set_max_value(1e+20)
		self.donor.set_plif_name("donor")

		l=array(model.penalty_coding_len_boundaries).flatten()
		p=array(model.penalty_coding_len_penalty).flatten()
		self.coding_len=Plif(len(l))
		self.coding_len.set_plif_limits(l)
		self.coding_len.set_plif_penalty(p)
		self.coding_len.set_min_value(min_exon_len)
		self.coding_len.set_max_value(max_len)
		self.coding_len.set_plif_name('coding_len')
		self.coding_len.set_transform_type("log(+1)")

		l=array(model.penalty_first_coding_len_boundaries).flatten()
		p=array(model.penalty_first_coding_len_penalty).flatten()
		self.first_coding_len=Plif(len(l))
		self.first_coding_len.set_plif_limits(l)
		self.first_coding_len.set_plif_penalty(p)
		self.first_coding_len.set_min_value(min_exon_len)
		self.first_coding_len.set_max_value(max_len)
		self.first_coding_len.set_plif_name("first_coding_len")
		self.first_coding_len.set_transform_type("log(+1)")

		l=array(model.penalty_last_coding_len_boundaries).flatten()
		p=array(model.penalty_last_coding_len_penalty).flatten()
		self.last_coding_len=Plif(len(l))
		self.last_coding_len.set_plif_limits(l)
		self.last_coding_len.set_plif_penalty(p)
		self.last_coding_len.set_min_value(min_exon_len)
		self.last_coding_len.set_max_value(max_len)
		self.last_coding_len.set_plif_name('last_coding_len')
		self.last_coding_len.set_transform_type("log(+1)")

		l=array(model.penalty_single_coding_len_boundaries).flatten()
		p=array(model.penalty_single_coding_len_penalty).flatten()
		self.single_coding_len=Plif(len(l))
		self.single_coding_len.set_plif_limits(l)
		self.single_coding_len.set_plif_penalty(p)
		self.single_coding_len.set_min_value(min_exon_len)
		self.single_coding_len.set_max_value(max_len)
		self.single_coding_len.set_plif_name('single_coding_len')
		self.single_coding_len.set_transform_type("log(+1)")

		l=array(model.penalty_intron_len_boundaries).flatten()
		p=array(model.penalty_intron_len_penalty).flatten()
		self.intron_len=Plif(len(l))
		self.intron_len.set_plif_limits(l)
		self.intron_len.set_plif_penalty(p)
		self.intron_len.set_min_value(min_intron_len)
		self.intron_len.set_max_value(max_len)
		self.intron_len.set_plif_name('intron_len')
		self.intron_len.set_transform_type("log(+1)")

		l=array(model.penalty_coding_boundaries).flatten()
		p=array(model.penalty_coding_penalty).flatten()
		self.coding=Plif(len(l))
		self.coding.set_use_svm(1)
		self.coding.set_plif_limits(l)
		self.coding.set_plif_penalty(p)
		self.coding.set_min_value(-1e+20)
		self.coding.set_max_value(1e+20)
		self.coding.set_plif_name('coding')

		l=array(model.penalty_coding2_boundaries).flatten()
		p=array(model.penalty_coding2_penalty).flatten()
		self.coding2=Plif(len(l))
		self.coding2.set_use_svm(3)
		self.coding2.set_plif_limits(l)
		self.coding2.set_plif_penalty(p)
		self.coding2.set_min_value(-1e+20)
		self.coding2.set_max_value(1e+20)
		self.coding2.set_plif_name('coding2')

		l=array(model.penalty_coding3_boundaries).flatten()
		p=array(model.penalty_coding3_penalty).flatten()
		self.coding3=Plif(len(l))
		self.coding3.set_use_svm(5)
		self.coding3.set_plif_limits(l)
		self.coding3.set_plif_penalty(p)
		self.coding3.set_min_value(-1e+20)
		self.coding3.set_max_value(1e+20)
		self.coding3.set_plif_name('coding3')

		l=array(model.penalty_coding4_boundaries).flatten()
		p=array(model.penalty_coding4_penalty).flatten()
		self.coding4=Plif(len(l))
		self.coding4.set_use_svm(7)
		self.coding4.set_plif_limits(l)
		self.coding4.set_plif_penalty(p)
		self.coding4.set_min_value(-1e+20)
		self.coding4.set_max_value(1e+20)
		self.coding4.set_plif_name('coding4')

		l=array(model.penalty_intron_boundaries).flatten()
		p=array(model.penalty_intron_penalty).flatten()
		self.intron=Plif(len(l))
		self.intron.set_use_svm(2)
		self.intron.set_plif_limits(l)
		self.intron.set_plif_penalty(p)
		self.intron.set_min_value(-1e+20)
		self.intron.set_max_value(1e+20)
		self.intron.set_plif_name('intron')

		l=array(model.penalty_intron2_boundaries).flatten()
		p=array(model.penalty_intron2_penalty).flatten()
		self.intron2=Plif(len(l))
		self.intron2.set_use_svm(4)
		self.intron2.set_plif_limits(l)
		self.intron2.set_plif_penalty(p)
		self.intron2.set_min_value(-1e+20)
		self.intron2.set_max_value(1e+20)
		self.intron2.set_plif_name('intron2')

		l=array(model.penalty_intron3_boundaries).flatten()
		p=array(model.penalty_intron3_penalty).flatten()
		self.intron3=Plif(len(l))
		self.intron3.set_use_svm(6)
		self.intron3.set_plif_limits(l)
		self.intron3.set_plif_penalty(p)
		self.intron3.set_min_value(-1e+20)
		self.intron3.set_max_value(1e+20)
		self.intron3.set_plif_name('intron3')

		l=array(model.penalty_intron4_boundaries).flatten()
		p=array(model.penalty_intron4_penalty).flatten()
		self.intron4=Plif(len(l))
		self.intron4.set_use_svm(8)
		self.intron4.set_plif_limits(l)
		self.intron4.set_plif_penalty(p)
		self.intron4.set_min_value(-1e+20)
		self.intron4.set_max_value(1e+20)
		self.intron4.set_plif_name('intron4')

		p=array(model.penalty_transitions_penalty).flatten()
		self.transitions=Plif(len(p))
		self.transitions.set_plif_penalty(p)
		self.transitions.set_min_value(-1e+20)
		self.transitions.set_max_value(1e+20)

		#create magic plifarrays
		self.first_coding_plif_array=PlifArray()
		self.first_coding_plif_array.add_plif(self.first_coding_len)
		self.first_coding_plif_array.add_plif(self.coding)
		self.first_coding_plif_array.add_plif(self.coding2)
		self.first_coding_plif_array.add_plif(self.coding3)
		self.first_coding_plif_array.add_plif(self.coding4)

		self.last_coding_plif_array=PlifArray()
		self.last_coding_plif_array.add_plif(self.last_coding_len)
		self.last_coding_plif_array.add_plif(self.coding)
		self.last_coding_plif_array.add_plif(self.coding2)
		self.last_coding_plif_array.add_plif(self.coding3)
		self.last_coding_plif_array.add_plif(self.coding4)

		self.coding_plif_array=PlifArray()
		self.coding_plif_array.add_plif(self.coding_len)
		self.coding_plif_array.add_plif(self.coding)
		self.coding_plif_array.add_plif(self.coding2)
		self.coding_plif_array.add_plif(self.coding3)
		self.coding_plif_array.add_plif(self.coding4)

		self.single_coding_plif_array=PlifArray()
		self.single_coding_plif_array.add_plif(self.single_coding_len)
		self.single_coding_plif_array.add_plif(self.coding)
		self.single_coding_plif_array.add_plif(self.coding2)
		self.single_coding_plif_array.add_plif(self.coding3)
		self.single_coding_plif_array.add_plif(self.coding4)

		self.intron_plif_array=PlifArray()
		self.intron_plif_array.add_plif(self.intron_len)
		self.intron_plif_array.add_plif(self.intron)
		self.intron_plif_array.add_plif(self.intron2)
		self.intron_plif_array.add_plif(self.intron3)
		self.intron_plif_array.add_plif(self.intron4)

		#finally create a single array with all the plifs
		self.plif_array=DynamicPlifArray()
		self.plif_array.append_element(self.acceptor)
		self.plif_array.append_element(self.donor)
		self.plif_array.append_element(self.first_coding_plif_array)
		self.plif_array.append_element(self.last_coding_plif_array)
		self.plif_array.append_element(self.coding_plif_array)
		self.plif_array.append_element(self.single_coding_plif_array)
		self.plif_array.append_element(self.intron_plif_array)

	def get_plif_array(self):
		return self.plif_array



""" Utilities for matplotlib examples """

import pylab
from numpy import ones, array, double, meshgrid, reshape, linspace, \
	concatenate, ravel, pi, sinc
from numpy.random import randn, rand
from modshogun import BinaryLabels, RegressionLabels, RealFeatures, SparseRealFeatures

QUITKEY='q'
NUM_EXAMPLES=100
DISTANCE=2

def quit (event):
	if event.key==QUITKEY or event.key==QUITKEY.upper():
		pylab.close()

def set_title (title):
	quitmsg=" (press '"+QUITKEY+"' to quit)"
	complete=title+quitmsg
	manager=pylab.get_current_fig_manager()

	# now we have to wrap the toolkit
	if hasattr(manager, 'window'):
		if hasattr(manager.window, 'setCaption'): # QT
			manager.window.setCaption(complete)
		if hasattr(manager.window, 'set_title'): # GTK
			manager.window.set_title(complete)
		elif hasattr(manager.window, 'title'): # TK
			manager.window.title(complete)


def get_realdata(positive=True):
	if positive:
		return randn(2, NUM_EXAMPLES)+DISTANCE
	else:
		return randn(2, NUM_EXAMPLES)-DISTANCE


def get_realfeatures(pos, neg):
	arr=array((pos, neg))
	features = concatenate(arr, axis=1)
	return RealFeatures(features)


def get_labels(raw=False, type='binary'):
	data = concatenate(array(
		(-ones(NUM_EXAMPLES, dtype=double), ones(NUM_EXAMPLES, dtype=double))
	))
	if raw:
		return data
	else:
		if type == 'binary':
			return BinaryLabels(data)
		if type == 'regression':
			return RegressionLabels(data)
		return None


def compute_output_plot_isolines(classifier, kernel=None, train=None, sparse=False, pos=None, neg=None, regression=False):
	size=100
	if pos is not None and neg is not None:
		x1_max=max(1.2*pos[0,:])
		x1_min=min(1.2*neg[0,:])
		x2_min=min(1.2*neg[1,:])
		x2_max=max(1.2*pos[1,:])
		x1=linspace(x1_min, x1_max, size)
		x2=linspace(x2_min, x2_max, size)
	else:
		x1=linspace(-5, 5, size)
		x2=linspace(-5, 5, size)

	x, y=meshgrid(x1, x2)

	dense=RealFeatures(array((ravel(x), ravel(y))))
	if sparse:
		test=SparseRealFeatures()
		test.obtain_from_simple(dense)
	else:
		test=dense

	if kernel and train:
		kernel.init(train, test)
	else:
		classifier.set_features(test)

	labels = None
	if regression:
		labels=classifier.apply().get_labels()
	else:
		labels=classifier.apply().get_values()
	z=labels.reshape((size, size))

	return x, y, z


def get_sinedata():
	x=4*rand(1, NUM_EXAMPLES)-DISTANCE
	x.sort()
	y=sinc(pi*x)+0.1*randn(1, NUM_EXAMPLES)

	return x, y


def compute_output_plot_isolines_sine(classifier, kernel, train, regression=False):
	x=4*rand(1, 500)-2
	x.sort()
	test=RealFeatures(x)
	kernel.init(train, test)

	if regression:
		y=classifier.apply().get_labels()
	else:
		y=classifier.apply().get_values()

	return x, y



#!/usr/bin/env python
import numpy
# create dense matrix A
A=numpy.array([[1,2,3],[4,0,0],[0,0,0],[0,5,0],[0,0,6],[9,9,9]], dtype=numpy.float64)

parameter_list=[[A]]
def features_sparse_modular (A):
	from scipy.sparse import csc_matrix
	from modshogun import SparseRealFeatures
	from numpy import array, float64, all

	# sparse representation X of dense matrix A
	# note, will work with types other than float64 too,
	# but requires recent scipy.sparse
	X=csc_matrix(A)
	#print(A)

	# create sparse shogun features from dense matrix A
	a=SparseRealFeatures(A)
	a_out=a.get_full_feature_matrix()
	#print(a_out)
	assert(all(a_out==A))
	#print(a_out)

	# create sparse shogun features from sparse matrix X
	a.set_sparse_feature_matrix(X)
	a_out=a.get_full_feature_matrix()
	#print(a_out)
	assert(all(a_out==A))

	# create sparse shogun features from sparse matrix X
	a=SparseRealFeatures(X)
	a_out=a.get_full_feature_matrix()
	#print(a_out)
	assert(all(a_out==A))

	# obtain (data,row,indptr) csc arrays of sparse shogun features
	z=csc_matrix(a.get_sparse_feature_matrix())
	z_out=z.todense()
	#print(z_out)
	assert(all(z_out==A))

if __name__=='__main__':
	print('Sparse Features')
	features_sparse_modular(*parameter_list[0])



CLASSIFIER=0
CLUSTERING=1
DISTANCE=2
DISTRIBUTION=3
KERNEL=4
PREPROC=5
REGRESSION=6


def get_name(catID):
	"""
	Get index to lookup the entity's name in the output data dictionary.

	@param catID ID of the category
	@return string index for name lookup in output data
	"""

	names={
		CLASSIFIER: 'classifier_name',
		CLUSTERING: 'clustering_name',
		DISTANCE: 'distance_name',
		DISTRIBUTION: 'distribution_name',
		KERNEL: 'kernel_name',
		PREPROC: 'preproc_name',
		REGRESSION: 'regression_name',
	}

	try:
		return names[catID]
	except KeyError:
		return ''


def get_as_string (catID):
	"""Returns the string representation of given category.

	@param catID ID of a category
	@return string of the category or empty string if ID was invalid
	"""

	strings={
		CLASSIFIER: 'classifier',
		CLUSTERING: 'clustering',
		DISTANCE: 'distance',
		DISTRIBUTION: 'distribution',
		KERNEL: 'kernel',
		PREPROC: 'preproc',
		REGRESSION: 'regression'
	}

	try:
		return strings[catID]
	except KeyError:
		return ''



#
# This program is free software you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation either version 3 of the License, or
# (at your option) any later version.
#
# Written (C) 2012 Heiko Strathmann, based on interactive_svm_demo by Christian
# Widmer which itself is based on PyQT Demo by Eli Bendersky
#

"""
Shogun Gaussian processes demo based on interactive SVM demo by Christian \
Widmer and Soeren Sonnenburg which itself is based on PyQT Demo by Eli Bendersky

Work to be done on parameter (e.g. kernel width) optimization.

Heiko Strathmann/Cameron Lai
License: GPLv3
"""
import sys, os, csv
import scipy as SP
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from numpy import *

import matplotlib
from matplotlib import mpl
from matplotlib.colorbar import make_axes, Colorbar
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as NavigationToolbar
from matplotlib.figure import Figure

from modshogun import *
from modshogun import *
from modshogun import *
import util

class Form(QMainWindow):
    def __init__(self, parent=None):
        super(Form, self).__init__(parent)
        self.setWindowTitle('SHOGUN interactive demo')

        self.series_list_model = QStandardItemModel()

        self.create_menu()
        self.create_main_frame()
        self.create_status_bar()
        self.create_toy_data()

        self.on_show()

    def on_show(self):
        self.axes.clear()
        self.axes.plot(self.x, self.y, 'ro')
        self.axes.set_xlim((self.xmin,self.xmax))
        self.axes.set_ylim((self.ymin,self.ymax))
        self.axes.grid(True)
        self.canvas.draw()
        self.fill_series_list(self.get_stats())


    def on_about(self):
        msg = __doc__
        QMessageBox.about(self, "About the demo", msg.strip())


    def fill_series_list(self, names):
        self.series_list_model.clear()

        for name in names:
            item = QStandardItem(name)
            item.setCheckState(Qt.Unchecked)
            item.setCheckable(False)
            self.series_list_model.appendRow(item)


    def onclick(self, event):
        print 'button=%d, x=%d, y=%d, xdata=%f, ydata=%f'%(event.button, event.x, event.y, event.xdata, event.ydata)
        x=SP.append(self.x, event.xdata)
        self.y=SP.append(self.y, event.ydata)
        self.x= x[:,SP.newaxis]
        self.on_show()
        self.status_text.setText("New data point: x=%f, y=%f"%(event.xdata, event.ydata))

    def create_menu(self):
        self.file_menu = self.menuBar().addMenu("&File")

        #load_action = self.create_action("&Load file",
        #    shortcut="Ctrl+L", slot=self.load_file, tip="Load a file")
        quit_action = self.create_action("&Quit", slot=self.close,
            shortcut="Ctrl+Q", tip="Close the application")

        #self.add_actions(self.file_menu,
        #    (load_action, None, quit_action))

        self.help_menu = self.menuBar().addMenu("&Help")
        about_action = self.create_action("&About",
            shortcut='F1', slot=self.on_about,
            tip='About the demo')

        self.add_actions(self.help_menu, (about_action,))

    def clear_data(self):
        self.x=SP.array([])
        self.y=SP.array([])
        self.xmin=-5
        self.xmax=5
        self.ymin=-5
        self.ymax=5
        self.on_show()
        self.status_text.setText("Data cleared")

    def enable_widgets(self):
        kernel_name = self.kernel_combo.currentText()
        if kernel_name == "Linear":
            self.sigma.setDisabled(True)
            self.degree.setDisabled(True)
        elif kernel_name == "Polynomial":
            self.sigma.setDisabled(True)
            self.degree.setEnabled(True)
        elif kernel_name == "Gaussian":
            self.sigma.setEnabled(True)
            self.degree.setDisabled(True)

    def get_stats(self):
        num_train = len(self.x)
        str_train = "num training points: %i" % num_train
        str_test = "num training points: %s" % self.nTest.text()

        return (str_train, str_test)

    def create_toy_data(self):
        #0. generate Toy-Data; just samples from a superposition of a sin + linear trend
        x = SP.arange(self.xmin,self.xmax,(self.xmax-self.xmin)/100.0)

        C = 2       #offset
        b = 0

        y  = b*x + C + float(self.sine_amplitude.text())*SP.sin(float(self.sine_freq.text())*x)
        #    dy = b   +     1*SP.cos(x)
        y += float(self.noise_level.text())*random.randn(y.shape[0])

        self.y=y-y.mean()
        self.x= x[:,SP.newaxis]
        self.on_show()

    def learn_kernel_width(self):
        root=ModelSelectionParameters();
        c1=ModelSelectionParameters("inference_method", inf);
        root.append_child(c1);

        c2 = ModelSelectionParameters("scale");
        c1.append_child(c2);
        c2.build_values(0.01, 4.0, R_LINEAR);
        c3 = ModelSelectionParameters("likelihood_model", likelihood);
        c1.append_child(c3);

        c4=ModelSelectionParameters("sigma");
        c3.append_child(c4);
        c4.build_values(0.001, 4.0, R_LINEAR);
        c5 =ModelSelectionParameters("kernel", SECF);
        c1.append_child(c5);

        c6 =ModelSelectionParameters("width");
        c5.append_child(c6);
        c6.build_values(0.001, 4.0, R_LINEAR);

        crit = GradientCriterion();

        grad=GradientEvaluation(gp, feat_train, labels, crit);

        grad.set_function(inf);

        gp.print_modsel_params();

        root.print_tree();

        grad_search=GradientModelSelection(root, grad);

        grad.set_autolock(0);

        best_combination=grad_search.select_model(1);

        self.sigma.setText("1.0")
        self.plot_gp()

    def plot_gp(self):
        feat_train = RealFeatures(self.x.T)
        labels = RegressionLabels(self.y)
        #[x,y]=self.data.get_data()
        #feat_train=RealFeatures(x.T)
        #labels=RegressionLabels(y)
        n_dimensions = 1

        kernel_name = self.kernel_combo.currentText()
        print "current kernel is %s" % (kernel_name)

        #new interface with likelihood parametres being decoupled from the covaraince function
        likelihood = GaussianLikelihood()
        #covar_parms = SP.log([2])
        #hyperparams = {'covar':covar_parms,'lik':SP.log([1])}

        # construct covariance function
        width=float(self.sigma.text())
        degree=int(self.degree.text())
        if kernel_name == "Linear":
            gk = LinearKernel(feat_train, feat_train)
            gk.set_normalizer(IdentityKernelNormalizer())
        elif kernel_name == "Polynomial":
            gk = PolyKernel(feat_train, feat_train, degree, True)
            gk.set_normalizer(IdentityKernelNormalizer())
        elif kernel_name == "Gaussian":
            gk = GaussianKernel(feat_train, feat_train, width)
        #SECF = GaussianKernel(feat_train, feat_train, width)
        #covar = SECF
        zmean = ZeroMean();
        inf = ExactInferenceMethod(gk, feat_train, zmean, labels, likelihood);

        inf.get_negative_marginal_likelihood()
        # location of unispaced predictions
        x_test = array([linspace(self.xmin,self.xmax, self.nTest.text())])
        feat_test=RealFeatures(x_test)

        gp = GaussianProcessRegression(inf)
        gp.train()
        covariance = gp.get_variance_vector(feat_test)
        predictions = gp.get_mean_vector(feat_test)

        #print "x_test"
        #print feat_test.get_feature_matrix()
        #print "mean predictions"
        #print predictions.get_labels()
        #print "covariances"
        #print covariance.get_labels()

        self.status_text.setText("Negative Log Marginal Likelihood = %f"%(inf.get_negative_marginal_likelihood()))


        self.axes.clear()
        self.axes.grid(True)
        self.axes.set_xlim((self.xmin,self.xmax))
        self.axes.set_ylim((self.ymin,self.ymax))
        self.axes.hold(True)
        x_test=feat_test.get_feature_matrix()[0]
        self.axes.plot(x_test, predictions, 'b-x')
        #self.axes.plot(x_test, labels.get_labels(), 'ro')
        self.axes.plot(self.x, self.y, 'ro')
        #self.axes.plot(feat_test.get_feature_matrix()[0], predictions.get_labels()-3*sqrt(covariance.get_labels()))
        #self.axes.plot(feat_test.get_feature_matrix()[0], predictions.get_labels()+3*sqrt(covariance.get_labels()))
        upper = predictions+3*sqrt(covariance)
        lower = predictions-3*sqrt(covariance)
        self.axes.fill_between(x_test, lower, upper, color='grey')
        self.axes.hold(False)
        self.canvas.draw()
        self.fill_series_list(self.get_stats())


    def create_main_frame(self):
        self.xmin=-5
        self.xmax=5
        self.ymin=-5
        self.ymax=5

        self.main_frame = QWidget()

        plot_frame = QWidget()

        self.dpi = 100
        self.fig = Figure((6.0, 6.0), dpi=self.dpi)
        self.canvas = FigureCanvas(self.fig)
        self.canvas.setParent(self.main_frame)

        cid = self.canvas.mpl_connect('button_press_event', self.onclick)
        self.axes = self.fig.add_subplot(111)
        self.cax = None
        #self.mpl_toolbar = NavigationToolbar(self.canvas, self.main_frame)

        self.kernel_combo = QComboBox()
        self.kernel_combo.insertItem(-1, "Gaussian")
        self.kernel_combo.insertItem(-1, "Polynomial")
        self.kernel_combo.insertItem(-1, "Linear")
        self.kernel_combo.maximumSize = QSize(300, 50)
        self.connect(self.kernel_combo, SIGNAL("currentIndexChanged(QString)"), self.enable_widgets)

        log_label = QLabel("Data points")
        self.series_list_view = QListView()
        self.series_list_view.setModel(self.series_list_model)

        self.sine_freq = QLineEdit()
        self.sine_freq.setText("1.0")
        self.sine_amplitude = QLineEdit()
        self.sine_amplitude.setText("1.0")
        self.sigma = QLineEdit()
        self.sigma.setText("1.2")
        self.degree = QLineEdit()
        self.degree.setText("2")
        self.noise_level = QLineEdit()
        self.noise_level.setText("1")
        self.nTest = QLineEdit()
        self.nTest.setText("100")

        spins_hbox = QHBoxLayout()
        spins_hbox.addWidget(QLabel('Sine data setting:     '))
        spins_hbox.addWidget(QLabel('Sine Freq.'))
        spins_hbox.addWidget(self.sine_freq)
        spins_hbox.addWidget(QLabel('Sine Amplitude'))
        spins_hbox.addWidget(self.sine_amplitude)
        spins_hbox.addWidget(QLabel('Noise Level'))
        spins_hbox.addWidget(self.noise_level)
        spins_hbox.addStretch(1)

        spins_hbox2 = QHBoxLayout()
        spins_hbox2.addWidget(QLabel('Kernel Setting:      '))
        spins_hbox2.addWidget(QLabel('Type'))
        spins_hbox2.addWidget(self.kernel_combo)
        spins_hbox2.addWidget(QLabel("Width"))
        spins_hbox2.addWidget(self.sigma)
        spins_hbox2.addWidget(QLabel("Degree"))
        spins_hbox2.addWidget(self.degree)
        spins_hbox2.addStretch(1)

        spins_hbox3 = QHBoxLayout()
        spins_hbox3.addWidget(QLabel('Test Setting:     '))
        spins_hbox3.addWidget(QLabel('Number of test points'))
        spins_hbox3.addWidget(self.nTest)
        spins_hbox3.addStretch(1)

        self.show_button = QPushButton("&Train GP")
        self.connect(self.show_button, SIGNAL('clicked()'), self.plot_gp)

        self.gen_sine_data_button = QPushButton("&Generate Sine Data")
        self.connect(self.gen_sine_data_button, SIGNAL('clicked()'), self.create_toy_data)

        self.clear_data_button = QPushButton("&Clear")
        self.connect(self.clear_data_button, SIGNAL('clicked()'), self.clear_data)

        self.learn_kernel_button = QPushButton("&Learn Kernel Width and train GP")
        self.connect(self.learn_kernel_button, SIGNAL('clicked()'), self.learn_kernel_width)

        left_vbox = QVBoxLayout()
        left_vbox.addWidget(self.canvas)
        #left_vbox.addWidget(self.mpl_toolbar)

        right0_vbox = QVBoxLayout()
        right0_vbox.addWidget(QLabel("Data Points"))
        right0_vbox.addWidget(self.series_list_view)
        #right0_vbox.addWidget(self.legend_cb)
        right0_vbox.addStretch(1)

        right2_vbox = QVBoxLayout()
        right2_vbox.addWidget(QLabel("Settings"))
        right2_vbox.addWidget(self.gen_sine_data_button)
        right2_vbox.addWidget(self.clear_data_button)
        right2_vbox.addWidget(self.show_button)
        #right2_vbox.addWidget(self.learn_kernel_button)
        right2_vbox.addLayout(spins_hbox)
        right2_vbox.addLayout(spins_hbox2)
        right2_vbox.addLayout(spins_hbox3)
        right2_vbox.addStretch(1)

        right_vbox = QHBoxLayout()
        right_vbox.addLayout(right0_vbox)
        right_vbox.addLayout(right2_vbox)

        hbox = QVBoxLayout()
        hbox.addLayout(left_vbox)
        hbox.addLayout(right_vbox)
        self.main_frame.setLayout(hbox)

        self.setCentralWidget(self.main_frame)
        self.enable_widgets()



    def create_status_bar(self):
        self.status_text = QLabel("")
        self.statusBar().addWidget(self.status_text, 1)

    def add_actions(self, target, actions):
        for action in actions:
            if action is None:
                target.addSeparator()
            else:
                target.addAction(action)

    def create_action(  self, text, slot=None, shortcut=None,
                        icon=None, tip=None, checkable=False,
                        signal="triggered()"):
        action = QAction(text, self)
        if icon is not None:
            action.setIcon(QIcon(":/%s.png" % icon))
        if shortcut is not None:
            action.setShortcut(shortcut)
        if tip is not None:
            action.setToolTip(tip)
            action.setStatusTip(tip)
        if slot is not None:
            self.connect(action, SIGNAL(signal), slot)
        if checkable:
            action.setCheckable(True)
        return action

def main():
    app = QApplication(sys.argv)
    form = Form()
    form.show()
    app.exec_()


if __name__ == "__main__":
    main()




"""
Test Distance
"""

from sg import sg
import util


def _evaluate (indata, prefix):
	dmatrix=sg('get_distance_matrix', 'TRAIN')
	dm_train=max(abs(indata['distance_matrix_train']-dmatrix).flat)

	dmatrix=sg('get_distance_matrix', 'TEST')
	dm_test=max(abs(indata['distance_matrix_test']-dmatrix).flat)

	return util.check_accuracy(
		indata[prefix+'accuracy'], dm_train=dm_train, dm_test=dm_test)


########################################################################
# public
########################################################################

def test (indata):
	prefix='distance_'
	try:
		util.set_features(indata, prefix)
	except NotImplementedError, e:
		print e
		return True

	util.set_and_train_distance(indata)

	return _evaluate(indata, prefix)




from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()

traindat=lm.load_numbers('../data/fm_train_real.dat')
testdat=lm.load_numbers('../data/fm_test_real.dat')
parameter_list=[[traindat,testdat,1.4,10],[traindat,testdat,1.5,11]]

def preproc_logplusone (fm_train_real=traindat,fm_test_real=testdat,
		 width=1.4,size_cache=10):

	sg('add_preproc', 'LOGPLUSONE')
	sg('set_kernel', 'CHI2', 'REAL', size_cache, width)

	sg('set_features', 'TRAIN', fm_train_real)
	sg('attach_preproc', 'TRAIN')
	km=sg('get_kernel_matrix', 'TRAIN')

	sg('set_features', 'TEST', fm_test_real)
	sg('attach_preproc', 'TEST')
	km=sg('get_kernel_matrix', 'TEST')
	return km

if __name__=='__main__':
	print('LogPlusOne')
	preproc_logplusone(*parameter_list[0])



#!/usr/bin/env python
import time
from modshogun import Time

parameter_list = [[5],[1.0]]
def library_time (sleep_secs):
	# measure wall clock time difference
	t=Time()
	time.sleep(sleep_secs)
	diff=t.cur_time_diff()

	# measure CPU time required
	cpu_diff=t.cur_runtime_diff_sec()

	# wall clock time should be above sleep_secs
	# but cpu time should be tiny
	#print diff, cpu_diff
	return diff>sleep_secs, cpu_diff<0.5

if __name__=='__main__':
	print('Time')
	library_time(*parameter_list[0])



#!/usr/bin/env python
from numpy import array,hstack
from numpy.random import seed, rand
from tools.load import LoadMatrix
lm=LoadMatrix()

traindat = lm.load_numbers('../data/fm_train_real.dat')
testdat = lm.load_numbers('../data/fm_test_real.dat')
label_traindat = lm.load_labels('../data/label_train_twoclass.dat')

parameter_list = [[traindat,testdat,label_traindat]]

def classifier_featureblock_logistic_regression (fm_train=traindat,fm_test=testdat,label_train=label_traindat):

	from modshogun import BinaryLabels, RealFeatures, IndexBlock, IndexBlockGroup, FeatureBlockLogisticRegression

	features = RealFeatures(hstack((traindat,traindat)))
	labels = BinaryLabels(hstack((label_train,label_train)))

	n_features = features.get_num_features()
	block_one = IndexBlock(0,n_features//2)
	block_two = IndexBlock(n_features//2,n_features)
	block_group = IndexBlockGroup()
	block_group.add_block(block_one)
	block_group.add_block(block_two)

	mtlr = FeatureBlockLogisticRegression(0.1,features,labels,block_group)
	mtlr.set_regularization(1) # use regularization ratio
	mtlr.set_tolerance(1e-2) # use 1e-2 tolerance
	mtlr.train()
	out = mtlr.apply().get_labels()

	return out

if __name__=='__main__':
	print('FeatureBlockLogisticRegression')
	classifier_featureblock_logistic_regression(*parameter_list[0])



from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()

traindat=lm.load_numbers('../data/fm_train_real.dat')
testdat=lm.load_numbers('../data/fm_test_real.dat')
parameter_list=[[traindat,testdat,11,1.2,1.3,10],[traindat,testdat,12,1.3,1.4,11]]

def kernel_sigmoid (fm_train_real=traindat,fm_test_real=testdat,
		 num_feats=11,gamma=1.2,coef0=1.3,size_cache=10):

	sg('set_features', 'TRAIN', fm_train_real)
	sg('set_features', 'TEST', fm_test_real)
	sg('set_kernel', 'SIGMOID', 'REAL', size_cache, gamma, coef0)
	km=sg('get_kernel_matrix', 'TRAIN')
	km=sg('get_kernel_matrix', 'TEST')
	return km

if __name__=='__main__':
	print('Sigmoid')
	kernel_sigmoid(*parameter_list[0])



#!/usr/bin/env python

from numpy import *

parameter_list = [[100, 2, 5,1.,1000,1,1], [100, 2, 5,1.,1000,1,2]]

def classifier_perceptron_modular (n=100, dim=2, distance=5,learn_rate=1.,max_iter=1000,num_threads=1,seed=1):
	from modshogun import RealFeatures, BinaryLabels
	from modshogun import Perceptron

	random.seed(seed)

	# produce some (probably) linearly separable training data by hand
	# Two Gaussians at a far enough distance
	X=array(random.randn(dim,n))+distance
	Y=array(random.randn(dim,n))-distance
	X_test=array(random.randn(dim,n))+distance
	Y_test=array(random.randn(dim,n))-distance
	label_train_twoclass=hstack((ones(n), -ones(n)))

	#plot(X[0,:], X[1,:], 'x', Y[0,:], Y[1,:], 'o')
	fm_train_real=hstack((X,Y))
	fm_test_real=hstack((X_test,Y_test))

	feats_train=RealFeatures(fm_train_real)
	feats_test=RealFeatures(fm_test_real)

	labels=BinaryLabels(label_train_twoclass)

	perceptron=Perceptron(feats_train, labels)
	perceptron.set_learn_rate(learn_rate)
	perceptron.set_max_iter(max_iter)
	# only guaranteed to converge for separable data
	perceptron.train()

	perceptron.set_features(feats_test)
	out_labels = perceptron.apply().get_labels()
	return perceptron, out_labels

if __name__=='__main__':
	print('Perceptron')
	classifier_perceptron_modular(*parameter_list[0])




#from matplotlib import rc
#rc('text', usetex=True)

fontsize = 16
contourFontsize = 12
showColorbar = False
xmin = -1
xmax = 1
ymin = -1.05
ymax = 1

import sys,os

import numpy
import shogun
from shogun.Kernel import GaussianKernel, LinearKernel, PolyKernel
from shogun.Features import RealFeatures, BinaryLabels
from shogun.Classifier import LibSVM

from numpy import arange
import matplotlib
from matplotlib import pylab
pylab.rcParams['contour.negative_linestyle'] = 'solid'

def features_from_file(fileName) :

    fileHandle = open(fileName)
    fileHandle.readline()
    features = []
    labels = []
    for line in fileHandle :
        tokens = line.split(',')
        labels.append(float(tokens[1]))
        features.append([float(token) for token in tokens[2:]])

    return RealFeatures(numpy.transpose(numpy.array(features))), features, BinaryLabels(numpy.array(labels,numpy.float))

def create_kernel(kname, features, kparam=None) :

    if kname == 'gauss' :
        kernel = GaussianKernel(features, features, kparam)
    elif kname == 'linear':
        kernel = LinearKernel(features, features)
    elif kname == 'poly' :
        kernel = PolyKernel(features, features, kparam, True, False)

    return kernel


def svm_train(kernel, labels, C1, C2=None):
    """Trains a SVM with the given kernel"""

    num_threads = 1

    kernel.io.disable_progress()
    svm = LibSVM(C1, kernel, labels)
    if C2:
        svm.set_C(C1, C2)
    svm.parallel.set_num_threads(num_threads)
    svm.io.disable_progress()
    svm.train()

    return svm

def svm_test(svm, kernel, features_train, features_test) :
    """predicts on the test examples"""

    kernel.init(features_train, features_test)
    output = svm.apply().get_labels()

    return output


def decision_boundary_plot(svm, features, vectors, labels, kernel, fileName = None, **args) :

    title = None
    if 'title' in args :
        title = args['title']
    xlabel = None
    if 'xlabel' in args :
        xlabel = args['xlabel']
    ylabel = None
    if 'ylabel' in args :
        ylabel = args['ylabel']
    fontsize = 'medium'
    if 'fontsize' in args :
        fontsize = args['fontsize']
    contourFontsize = 10
    if 'contourFontsize' in args :
        contourFontsize = args['contourFontsize']
    showColorbar = True
    if 'showColorbar' in args :
        showColorbar = args['showColorbar']
    show = True
    if fileName is not None :
        show = False
    if 'show' in args :
        show = args['show']


    # setting up the grid
    delta = 0.005
    x = arange(xmin, xmax, delta)
    y = arange(ymin, ymax, delta)

    Z = numpy.zeros((len(x), len(y)), numpy.float_)
    gridX = numpy.zeros((len(x) *len(y), 2), numpy.float_)
    n = 0
    for i in range(len(x)) :
        for j in range(len(y)) :
            gridX[n][0] = x[i]
            gridX[n][1] = y[j]
            n += 1

    if kernel.get_name() == 'Linear' and 'customwandb' in args:
        kernel.init_optimization_svm(svm)
        b=svm.get_bias()
        w=kernel.get_w()
        kernel.set_w(args['customwandb'][0])
        svm.set_bias(args['customwandb'][1])

    if kernel.get_name() == 'Linear' and 'drawarrow' in args:
        kernel.init_optimization_svm(svm)
        b=svm.get_bias()
        w=kernel.get_w()
        s=1.0/numpy.dot(w,w)/1.17
        pylab.arrow(0,-b/w[1], w[0]*s,s*w[1], width=0.01, fc='#dddddd', ec='k')
    grid_features = RealFeatures(numpy.transpose(gridX))
    results = svm_test(svm, kernel, features, grid_features)

    n = 0
    for i in range(len(x)) :
        for j in range(len(y)) :
            Z[i][j] = results[n]
            n += 1

    cdict = {'red'  :((0.0, 0.6, 0.6),(0.5, 0.8, 0.8),(1.0, 1.0, 1.0)),
             'green':((0.0, 0.6, 0.6),(0.5, 0.8, 0.8),(1.0, 1.0, 1.0)),
             'blue' :((0.0, 0.6, 0.6),(0.5, 0.8, 0.8),(1.0, 1.0, 1.0)),
             }
    my_cmap = matplotlib.colors.LinearSegmentedColormap('lightgray',cdict,256)
    im = pylab.imshow(numpy.transpose(Z),
                      interpolation='bilinear', origin='lower',
                      cmap=my_cmap, extent=(xmin,xmax,ymin,ymax) )

    if 'decisionboundaryonly' in args:
        C1 = pylab.contour(numpy.transpose(Z),
                [0],
                origin='lower',
                linewidths=(3),
                colors = ['k'],
                extent=(xmin,xmax,ymin,ymax))
    else:
        C1 = pylab.contour(numpy.transpose(Z),
                [-1,0,1],
                origin='lower',
                linewidths=(1,3,1),
                colors = ['k','k'],
                extent=(xmin,xmax,ymin,ymax))

        pylab.clabel(C1,
                inline=1,
                fmt='%1.1f',
                fontsize=contourFontsize)

    # plot the data
    lab=labels.get_labels()
    vec=numpy.array(vectors)
    idx=numpy.where(lab==-1)[0]
    pylab.scatter(vec[idx,0], vec[idx,1], s=300, c='#4444ff', marker='o', alpha=0.8, zorder=100)
    idx=numpy.where(lab==+1)[0]
    pylab.scatter(vec[idx,0], vec[idx,1], s=500, c='#ff4444', marker='s', alpha=0.8, zorder=100)

    # plot SVs
    if not 'decisionboundaryonly' in args:
        training_outputs = svm_test(svm, kernel, features, features)
        sv_idx=numpy.where(abs(training_outputs)<=1.01)[0]
        pylab.scatter(vec[sv_idx,0], vec[sv_idx,1], s=100, c='k', marker='o', alpha=0.8, zorder=100)

    if 'showmovedpoint' in args:
        x=-0.779838709677
        y=-0.1375
        pylab.scatter([x], [y], s=300, c='#4e4e61', marker='o', alpha=1, zorder=100, edgecolor='#454548')
        pylab.arrow(x,y-0.1, 0, -0.8/1.5, width=0.01, fc='#dddddd', ec='k')
        #pylab.show()


    if title is not None :
        pylab.title(title, fontsize=fontsize)
    if ylabel:
        pylab.ylabel(ylabel,fontsize=fontsize)
    if xlabel:
        pylab.xlabel(xlabel,fontsize=fontsize)
    if showColorbar :
        pylab.colorbar(im)

    # colormap:
    pylab.hot()
    if fileName is not None :
        pylab.savefig(fileName)
    if show :
        pylab.show()

def add_percent_ticks():
    ticks=pylab.getp(pylab.gca(),'xticks')
    ticklabels=len(ticks)*['']
    ticklabels[0]='0%'
    ticklabels[-1]='100%'
    pylab.setp(pylab.gca(), xticklabels=ticklabels)

    pylab.setp(pylab.gca(), yticklabels=['0%','100%'])
    ticks=pylab.getp(pylab.gca(),'yticks')
    ticklabels=len(ticks)*['']
    #ticklabels[0]='0%'
    ticklabels[-1]='100%'
    pylab.setp(pylab.gca(), yticklabels=ticklabels)

    xticklabels = pylab.getp(pylab.gca(), 'xticklabels')
    yticklabels = pylab.getp(pylab.gca(), 'yticklabels')
    pylab.setp(xticklabels, fontsize=fontsize)
    pylab.setp(yticklabels, fontsize=fontsize)



def create_figures(extension = 'pdf', directory = '../../tex/figures') :

    if extension[0] != '.' :
        extension = '.' + extension

    dpi=90

    # data and linear decision boundary
    features,vectors,labels = features_from_file('data/small_gc_toy.data')

    kernel = create_kernel('linear', features)
    svm = svm_train(kernel, labels, 0.7)

    pylab.figure(figsize=(8,6), dpi=dpi)
    decision_boundary_plot(svm, features, vectors, labels, kernel,
            fontsize=fontsize, contourFontsize=contourFontsize,
            title="Linear Separation", customwandb=(numpy.array([-0.05, -1.0]), -0.3),
            ylabel="GC Content Before 'AG'",xlabel="GC Content After 'AG'",
            show=False, showColorbar=showColorbar, decisionboundaryonly=True)
    add_percent_ticks()

    pylab.savefig(os.path.join(directory, 'data_and_linear_classifier' + extension))
    pylab.close()
#####################################################################################
    # data and svm decision boundary
    features,vectors,labels = features_from_file('data/small_gc_toy.data')

    kernel = create_kernel('linear', features)
    svm = svm_train(kernel, labels, 100)

    pylab.figure(figsize=(8,6), dpi=dpi)
    decision_boundary_plot(svm, features, vectors, labels, kernel,
            fontsize=fontsize, contourFontsize=contourFontsize,
            title="Maximum Margin Separation", drawarrow=True,
            ylabel="GC Content Before 'AG'",xlabel="GC Content After 'AG'",
            show=False, showColorbar=showColorbar)
    add_percent_ticks()

    pylab.savefig(os.path.join(directory, 'data_and_svm_classifier' + extension))
    pylab.close()
#####################################################################################


    # the effect of C on the decision surface:
    features,vectors,labels = features_from_file('data/small_gc_toy_outlier.data')

    pylab.figure(figsize=(16,6), dpi=dpi)
    pylab.subplot(121)
    kernel = create_kernel('linear', features)
    svm = svm_train(kernel, labels, 200)
    decision_boundary_plot(svm, features, vectors, labels, kernel,
            title = 'Soft-Margin with C=200', ylabel="GC Content Before 'AG'",
            xlabel="GC Content After 'AG'", fontsize=fontsize,
            contourFontsize=contourFontsize, show=False, showmovedpoint=True,
            showColorbar=showColorbar)
    add_percent_ticks()

    pylab.subplot(122)
    kernel = create_kernel('linear', features)
    svm = svm_train(kernel, labels, 2)
    decision_boundary_plot(svm, features, vectors, labels, kernel,
            title = 'Soft-Margin with C=2',
            ylabel="GC Content Before 'AG'",xlabel="GC Content After 'AG'",
            fontsize=fontsize, contourFontsize=contourFontsize, show=False, showColorbar=showColorbar)
    add_percent_ticks()
    #pylab.subplots_adjust(bottom=0.05, top=0.95)

    pylab.savefig(os.path.join(directory, 'effect_of_c' + extension))
    pylab.close()
####################################################################################

    # playing with nonlinear data:
    # the effect of kernel parameters

    features,vectors,labels = features_from_file('data/small_gc_toy_outlier.data')
    pylab.figure(figsize=(24,6), dpi=dpi)
    pylab.subplot(131)
    kernel = create_kernel('linear', features)
    svm = svm_train(kernel, labels, 100)
    decision_boundary_plot(svm, features, vectors, labels, kernel,
            title = 'Linear Kernel',
            ylabel="GC Content Before 'AG'",
            fontsize=fontsize, contourFontsize=contourFontsize, show=False,showColorbar=showColorbar)
    add_percent_ticks()

    pylab.subplot(132)
    kernel = create_kernel('poly', features, 2)
    svm = svm_train(kernel, labels, 100)
    decision_boundary_plot(svm, features, vectors, labels, kernel,
            title='Polynomial Kernel d=2',
            xlabel="GC Content After 'AG'",
            fontsize=fontsize, contourFontsize=contourFontsize, show=False,showColorbar=showColorbar)
    add_percent_ticks()

    pylab.subplot(133)
    kernel = create_kernel('poly', features, 5)
    svm = svm_train(kernel, labels, 10)
    decision_boundary_plot(svm, features, vectors, labels, kernel,
            title='Polynomial Kernel d=5',
            fontsize=fontsize, contourFontsize=contourFontsize, show=False,showColorbar=showColorbar)
    add_percent_ticks()
    #pylab.subplots_adjust(bottom=0.05, top=0.95)

    pylab.savefig(os.path.join(directory, 'params_polynomial' + extension))
    pylab.close()
####################################################################################

    #effects of sigma
    pylab.figure(figsize=(24,6), dpi=dpi)
    pylab.subplot(131)
    gamma = 0.1
    sigma = 20.0
    kernel = create_kernel('gauss', features, sigma)
    svm = svm_train(kernel, labels, 100)
    decision_boundary_plot(svm, features, vectors, labels, kernel,
            title='Gaussian Kernel Sigma=20',
            ylabel="GC Content Before 'AG'",
            fontsize=fontsize, contourFontsize=contourFontsize, show=False,showColorbar=showColorbar)
    add_percent_ticks()

    pylab.subplot(132)
    sigma = 1.0
    kernel = create_kernel('gauss', features, sigma)
    svm = svm_train(kernel, labels, 100)
    decision_boundary_plot(svm, features, vectors, labels, kernel,
            title='Gaussian Kernel Sigma=1',
            xlabel="GC Content After 'AG'",
            fontsize=fontsize, contourFontsize=contourFontsize, show=False,showColorbar=showColorbar)
    add_percent_ticks()

    pylab.subplot(133)
    sigma = 0.05
    kernel = create_kernel('gauss', features, sigma)
    svm = svm_train(kernel, labels, 100)
    decision_boundary_plot(svm, features, vectors, labels, kernel,
            title='Gaussian Kernel Sigma=0.05',
            fontsize=fontsize, contourFontsize=contourFontsize, show=False,showColorbar=showColorbar)
    add_percent_ticks()

    #pylab.subplots_adjust(bottom=0.05, top=0.95)

    pylab.savefig(os.path.join(directory, 'params_gaussian' + extension))
    pylab.close()
####################################################################################

if __name__ == '__main__' :

    extension = 'pdf'
    if len(sys.argv) > 1 :
        extension = sys.argv[1]
    pylab.ioff()
    create_figures(extension)



#
# This program is free software you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation either version 3 of the License, or
# (at your option) any later version.
#
# Written (W) 2013 Roman Votyakov
#

from pylab import *
from numpy import *
from itertools import *

def generate_toy_data(n_train=100, mean_a=asarray([0, 0]), std_dev_a=1.0, mean_b=3, std_dev_b=0.5):

    # positive examples are distributed normally
    X1 = (random.randn(n_train, 2)*std_dev_a+mean_a).T

    # negative examples have a "ring"-like form
    r = random.randn(n_train)*std_dev_b+mean_b
    angle = random.randn(n_train)*2*pi
    X2 = array([r*cos(angle)+mean_a[0], r*sin(angle)+mean_a[1]])

    # stack positive and negative examples in a single array
    X_train = hstack((X1,X2))

    # label positive examples with +1, negative with -1
    y_train = zeros(n_train*2)
    y_train[:n_train] = 1
    y_train[n_train:] = -1

    return [X_train, y_train]

def gaussian_process_binary_classification_laplace(X_train, y_train, n_test=50):

    # import all necessary modules from Shogun (some of them require Eigen3)
    try:
        from modshogun import RealFeatures, BinaryLabels, GaussianKernel, \
            LogitLikelihood, ProbitLikelihood, ZeroMean, LaplacianInferenceMethod, \
            EPInferenceMethod, GaussianProcessBinaryClassification
    except ImportError:
        print('Eigen3 needed for Gaussian Processes')
        return

    # convert training data into Shogun representation
    train_features = RealFeatures(X_train)
    train_labels = BinaryLabels(y_train)

    # generate all pairs in 2d range of testing data
    x1 = linspace(X_train[0,:].min()-1, X_train[0,:].max()+1, n_test)
    x2 = linspace(X_train[1,:].min()-1, X_train[1,:].max()+1, n_test)
    X_test = asarray(list(product(x1, x2))).T

    # convert testing features into Shogun representation
    test_features = RealFeatures(X_test)

    # create Gaussian kernel with width = 2.0
    kernel = GaussianKernel(10, 2.0)

    # create zero mean function
    mean = ZeroMean()

    # you can easily switch between probit and logit likelihood models
    # by uncommenting/commenting the following lines:

    # create probit likelihood model
    # lik = ProbitLikelihood()

    # create logit likelihood model
    lik = LogitLikelihood()

    # you can easily switch between Laplace and EP approximation by
    # uncommenting/commenting the following lines:

    # specify Laplace approximation inference method
    # inf = LaplacianInferenceMethod(kernel, train_features, mean, train_labels, lik)

    # specify EP approximation inference method
    inf = EPInferenceMethod(kernel, train_features, mean, train_labels, lik)

    # create and train GP classifier, which uses Laplace approximation
    gp = GaussianProcessBinaryClassification(inf)
    gp.train()

    # get probabilities p(y*=1|x*) for each testing feature x*
    p_test = gp.get_probabilities(test_features)

    # create figure
    figure()
    title('Training examples, predictive probability and decision boundary')

    # plot training data
    plot(X_train[0, argwhere(y_train == 1)], X_train[1, argwhere(y_train == 1)], 'ro')
    plot(X_train[0, argwhere(y_train == -1)], X_train[1, argwhere(y_train == -1)], 'bo')

    # plot decision boundary
    contour(x1, x2, reshape(p_test, (n_test, n_test)), levels=[0.5], colors=('black'))

    # plot probabilities
    pcolor(x1, x2, reshape(p_test, (n_test, n_test)))

    # show color bar
    colorbar()

    # show figure
    show()

if __name__=='__main__':
    [X_train, y_train] = generate_toy_data()
    gaussian_process_binary_classification_laplace(X_train, y_train)



#!/usr/bin/env python
from tools.load import LoadMatrix
from numpy import where

lm=LoadMatrix()
traindat = lm.load_numbers('../data/fm_train_real.dat')
testdat = lm.load_numbers('../data/fm_test_real.dat')

parameter_list=[[traindat,testdat, 2.0],[traindat,testdat, 3.0]]

def kernel_tstudent_modular (fm_train_real=traindat,fm_test_real=testdat, degree=2.0):
	from modshogun import RealFeatures
	from modshogun import TStudentKernel
	from modshogun import EuclideanDistance

	feats_train=RealFeatures(fm_train_real)
	feats_test=RealFeatures(fm_test_real)

	distance=EuclideanDistance(feats_train, feats_train)

	kernel=TStudentKernel(feats_train, feats_train, degree, distance)
	km_train=kernel.get_kernel_matrix()

	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	return km_train,km_test,kernel


if __name__=='__main__':
	print('TStudent')
	kernel_tstudent_modular(*parameter_list[0])



#!/usr/bin/env python
from modshogun import StreamingVwFile
from modshogun import T_SVMLIGHT
from modshogun import StreamingVwFeatures
from modshogun import VowpalWabbit

parameter_list=[[None]]

def streaming_vw_modular (dummy):
	"""Runs the VW algorithm on a toy dataset in SVMLight format."""

	# Open the input file as a StreamingVwFile
	input_file = StreamingVwFile("../data/fm_train_sparsereal.dat")

	# Tell VW that the file is in SVMLight format
	# Supported types are T_DENSE, T_SVMLIGHT and T_VW
	input_file.set_parser_type(T_SVMLIGHT)

	## Create a StreamingVwFeatures object, `True' indicating the examples are labelled
	#features = StreamingVwFeatures(input_file, True, 1024)

	## Create a VW object from the features
	#vw = VowpalWabbit(features)

	## Train
	#vw.train()

	##return vw

if __name__ == "__main__":
	streaming_vw_modular(*parameter_list[0])
